from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GCPAccessAttemptsViolatingVPCServiceControls(Rule):
    default_description = "An access attempt violating VPC service controls (such as Perimeter controls) has been made."
    display_name = "GCP Access Attempts Violating VPC Service Controls"
    default_reference = "https://cloud.google.com/vpc-service-controls/docs/troubleshooting#debugging"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.Access.Attempts.Violating.VPC.Service.Controls-prototype"

    def rule(self, event):
        severity = event.get("severity", "")
        status_code = event.deep_get("protoPayload", "status", "code", default="")
        violation_types = event.deep_walk("protoPayload", "status", "details", "violations", "type", default=[])
        if all([severity == "ERROR", status_code == 7, "VPC_SERVICE_CONTROLS" in violation_types]):
            return True
        return False

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        method = event.deep_get("protoPayload", "methodName", default="<METHOD_NOT_FOUND>")
        return f"GCP: [{actor}] performed a [{method}] request that violates VPC Service Controls"

    tests = [
        RuleTest(
            name="Other Event",
            expected_result=False,
            log={
                "insertId": "12345",
                "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "first": True,
                    "id": "operation-abcdefg-1234567",
                    "last": True,
                    "producer": "compute.googleapis.com",
                },
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-08 18:52:52.114",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-08 18:54:14.595",
                "p_row_id": "5e7586fcbb73fdeed985ebeb16bd0c",
                "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
                "p_source_label": "gcplogsource",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user1@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "compute.subnetworks.update",
                            "resourceAttributes": {
                                "name": "projects/test-project/regions/us-central1/subnetworks/default",
                                "service": "compute",
                                "type": "compute.subnetworks",
                            },
                        },
                    ],
                    "methodName": "v1.compute.subnetworks.patch",
                    "request": {
                        "@type": "type.googleapis.com/compute.subnetworks.patch",
                        "fingerprint": "/�/��\x03��",
                        "logConfig": {"enable": True},
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {
                            "auth": {},
                            "reason": "8uSywAYQGg5Db2xpc2V1bSBGbG93cw",
                            "time": "2023-03-08T18:52:52.558899Z",
                        },
                    },
                    "resourceName": "projects/test-project/regions/us-central1/subnetworks/default",
                    "response": {
                        "@type": "type.googleapis.com/operation",
                        "endTime": "2023-03-08T10:52:52.510-08:00",
                        "id": "9876543210",
                        "insertTime": "2023-03-08T10:52:52.509-08:00",
                        "name": "operation-abcdefg-1234567",
                        "operationType": "compute.subnetworks.patch",
                        "progress": "100",
                        "region": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1",
                        "selfLink": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/operations/operation-abcdefg-1234567",
                        "selfLinkWithId": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/operations/9876543210",
                        "startTime": "2023-03-08T10:52:52.510-08:00",
                        "status": "DONE",
                        "targetId": "567854321",
                        "targetLink": "https://www.googleapis.com/compute/v1/projects/test-project/regions/us-central1/subnetworks/default",
                        "user": "user1@company.io",
                    },
                    "serviceName": "compute.googleapis.com",
                },
                "receiveTimestamp": "2023-03-08 18:52:52.72",
                "resource": {
                    "labels": {
                        "location": "us-central1",
                        "project_id": "test-project",
                        "subnetwork_id": "567854321",
                        "subnetwork_name": "default",
                    },
                    "type": "gce_subnetwork",
                },
                "severity": "NOTICE",
                "timestamp": "2023-03-08 18:52:52.114",
            },
        ),
        RuleTest(
            name="VPC control violation",
            expected_result=True,
            log={
                "insertId": "13ogcded7jh2",
                "insertid": "15wr7lbb6j",
                "logName": "projects/gcpproject/logs/cloudaudit.googleapis.com%2Fpolicy",
                "logname": "projects/gcpproject/logs/cloudaudit.googleapis.com%2Factivity",
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-09 10:53:14.929",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-09 10:54:14.363",
                "p_row_id": "7ad218d42253b7e6f78cc0ed16be37",
                "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
                "p_source_label": "gcplogsource",
                "protoPayload": {
                    "at_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user1@serviceaccount.gcp.com"},
                    "metadata": {
                        "at_type": "type.googleapis.com/google.cloud.audit.VpcServiceControlAuditMetadata",
                        "deviceState": "Unknown",
                        "ingressViolations": [
                            {
                                "servicePerimeter": "accessPolicies/123456789012/servicePerimeters/test_perimeter",
                                "targetResource": "projects/197946410614",
                                "targetResourcePermissions": ["NO_PERMISSIONS"],
                            },
                        ],
                        "resourceNames": ["projects/_/buckets/test-restricted-bucket/objects/test1.txt"],
                        "securityPolicyInfo": {
                            "organizationId": "645568414902",
                            "servicePerimeterName": "accessPolicies/123456789012/servicePerimeters/test_perimeter",
                        },
                        "violationReason": "NO_MATCHING_ACCESS_LEVEL",
                        "vpcServiceControlsUniqueId": "gBc-wuGVCapNMnTUePoHos_VyJmr3CsMKlr48kVa4b6XpsT_OWKRng",
                    },
                    "methodName": "google.storage.objects.get",
                    "requestMetadata": {"callerIp": "1.2.3.4", "destinationAttributes": {}, "requestAttributes": {}},
                    "resourceName": "projects/197946410614",
                    "serviceName": "storage.googleapis.com",
                    "status": {
                        "code": 7,
                        "details": [
                            {
                                "at_type": "type.googleapis.com/google.rpc.PreconditionFailure",
                                "violations": [
                                    {
                                        "description": "gBc-wuGVCapNMnTUePoHos_VyJmr3CsMKlr48kVa4b6XpsT_OWKRng",
                                        "type": "VPC_SERVICE_CONTROLS",
                                    },
                                    {
                                        "description": "gCc-wuJa334DJ9940ssdiw_V8400skgjj3912500sldgjzh_LGJANr",
                                        "type": "OTHER_CONTROL_VIOLATION",
                                    },
                                ],
                            },
                        ],
                        "message": "Request is prohibited by organization's policy. vpcServiceControlsUniqueIdentifier: gBc-wuGVCapNMnTUePoHos_VyJmr3CsMKlr48kVa4b6XpsT_OWKRng",
                    },
                },
                "receiveTimestamp": "2023-03-09T16:28:42.567340480Z",
                "resource": {
                    "labels": {
                        "method": "google.storage.objects.get",
                        "project_id": "gcpproject",
                        "service": "storage.googleapis.com",
                    },
                    "type": "audited_resource",
                },
                "severity": "ERROR",
                "timestamp": "2023-03-09T16:28:40.890430163Z",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.okta import okta_alert_context


@panther_managed
class OktaAPIKeyRevoked(Rule):
    id = "Okta.APIKeyRevoked-prototype"
    display_name = "Okta API Key Revoked"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tags = ["Identity & Access Management", "Okta"]
    default_severity = Severity.INFO
    default_description = "A user has revoked an API Key in Okta"
    default_reference = "https://help.okta.com/en/prod/Content/Topics/Security/API.htm"
    default_runbook = "Validate this action was authorized."
    summary_attributes = ["eventType", "severity", "displayMessage", "p_any_ip_addresses"]

    def rule(self, event):
        return (
            event.get("eventType", None) == "system.api_token.revoke"
            and event.deep_get("outcome", "result") == "SUCCESS"
        )

    def title(self, event):
        target = event.get("target", [{}])
        key_name = target[0].get("displayName", "MISSING DISPLAY NAME") if target else "MISSING TARGET"
        return f"{event.deep_get('actor', 'displayName')} <{event.deep_get('actor', 'alternateId')}>revoked API key - <{key_name}>"

    def alert_context(self, event):
        return okta_alert_context(event)

    tests = [
        RuleTest(
            name="API Key Revoked",
            expected_result=True,
            log={
                "uuid": "2a992f80-d1ad-4f62-900e-8c68bb72a21b",
                "published": "2021-01-08 21:28:34.875",
                "eventType": "system.api_token.revoke",
                "version": "0",
                "severity": "INFO",
                "legacyEventType": "api.token.revoke",
                "displayMessage": "Revoke API token",
                "actor": {
                    "alternateId": "user@example.com",
                    "displayName": "Test User",
                    "id": "00u3q14ei6KUOm4Xi2p4",
                    "type": "User",
                },
                "outcome": {"result": "SUCCESS"},
                "request": {},
                "debugContext": {},
                "target": [
                    {
                        "id": "00Tpki36zlWjhjQ1u2p4",
                        "type": "Token",
                        "alternateId": "unknown",
                        "displayName": "test_key",
                        "details": None,
                    },
                ],
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class PushSecurityNewSaaSAccountCreated(Rule):
    id = "Push.Security.New.SaaS.Account.Created-prototype"
    display_name = "Push Security New SaaS Account Created"
    log_types = [LogType.PUSH_SECURITY_ENTITIES]
    default_severity = Severity.INFO

    def rule(self, event):
        if event.get("object") != "ACCOUNT":
            return False
        if event.get("type") == "CREATE":
            return True
        return False

    def title(self, event):
        app_type = event.deep_get("new", "appType")
        new_email = event.deep_get("new", "email")
        return f"New account on {app_type} created by {new_email}"

    tests = [
        RuleTest(
            name="Account Update",
            expected_result=False,
            log={
                "id": "c478966c-f927-411c-b919-179832d3d50c",
                "new": {
                    "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "appType": "ATLASSIAN",
                    "creationTimestamp": 1698064423.0,
                    "email": "john.hill@example.com",
                    "employeeId": "72d0347a-2663-4ef5-b1c5-df39163f1603",
                    "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                    "lastUsedTimestamp": 1698669168.0,
                    "loginMethods": {
                        "oidcLogin": "GOOGLE_WORKSPACE",
                        "oktaSwaLogin": True,
                        "passwordLogin": True,
                        "samlLogin": "OKTA",
                        "vendorSsoLogin": "GOOGLE_WORKSPACE",
                    },
                    "mfaMethods": [
                        "APP_TOTP",
                        "PUSH_NOTIFICATION",
                        "EMAIL_OTP",
                        "U2F",
                        "HARDWARE_TOTP",
                        "PHONE_CALL",
                        "SMS_OTP",
                        "APP_PASSWORD",
                        "GRID_CARD",
                        "EXTERNAL_PROVIDER",
                        "BACKUP_CODES",
                        "WEBAUTHN",
                    ],
                    "mfaRegistered": True,
                    "passwordId": "4c13674f-e88a-4411-bfa2-53a70468a898",
                },
                "object": "ACCOUNT",
                "old": {
                    "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "appType": "ATLASSIAN",
                    "creationTimestamp": 1698064423.0,
                    "email": "john.hill@example.com",
                    "employeeId": "72d0347a-2663-4ef5-b1c5-df39163f1603",
                    "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                    "lastUsedTimestamp": 1698669168.0,
                    "loginMethods": {
                        "oidcLogin": "GOOGLE_WORKSPACE",
                        "oktaSwaLogin": True,
                        "passwordLogin": True,
                        "samlLogin": "OKTA",
                        "vendorSsoLogin": "GOOGLE_WORKSPACE",
                    },
                    "mfaMethods": [
                        "APP_TOTP",
                        "PUSH_NOTIFICATION",
                        "EMAIL_OTP",
                        "U2F",
                        "HARDWARE_TOTP",
                        "PHONE_CALL",
                        "SMS_OTP",
                        "APP_PASSWORD",
                        "GRID_CARD",
                        "EXTERNAL_PROVIDER",
                        "BACKUP_CODES",
                        "WEBAUTHN",
                    ],
                    "mfaRegistered": True,
                    "passwordId": "4c13674f-e88a-4411-bfa2-53a70468a898",
                },
                "timestamp": 1698604061.0,
                "type": "UPDATE",
                "version": "1",
            },
        ),
        RuleTest(
            name="New Account",
            expected_result=True,
            log={
                "id": "c478966c-f927-411c-b919-179832d3d50c",
                "new": {
                    "appId": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "appType": "ATLASSIAN",
                    "creationTimestamp": 1698064423.0,
                    "email": "john.hill@example.com",
                    "employeeId": "72d0347a-2663-4ef5-b1c5-df39163f1603",
                    "id": "d6a32ba5-0532-4a66-8137-48cdf409c972",
                    "lastUsedTimestamp": 1698669168.0,
                    "loginMethods": {
                        "oidcLogin": "GOOGLE_WORKSPACE",
                        "oktaSwaLogin": True,
                        "passwordLogin": True,
                        "samlLogin": "OKTA",
                        "vendorSsoLogin": "GOOGLE_WORKSPACE",
                    },
                    "mfaMethods": [
                        "APP_TOTP",
                        "PUSH_NOTIFICATION",
                        "EMAIL_OTP",
                        "U2F",
                        "HARDWARE_TOTP",
                        "PHONE_CALL",
                        "SMS_OTP",
                        "APP_PASSWORD",
                        "GRID_CARD",
                        "EXTERNAL_PROVIDER",
                        "BACKUP_CODES",
                        "WEBAUTHN",
                    ],
                    "mfaRegistered": True,
                    "passwordId": "4c13674f-e88a-4411-bfa2-53a70468a898",
                },
                "object": "ACCOUNT",
                "old": None,
                "timestamp": 1698604061.0,
                "type": "CREATE",
                "version": "1",
            },
        ),
    ]



from pypanther.rules.auth0.auth0_cic_credential_stuffing import Auth0CICCredentialStuffing as Auth0CICCredentialStuffing
from pypanther.rules.auth0.auth0_custom_role_created import Auth0CustomRoleCreated as Auth0CustomRoleCreated
from pypanther.rules.auth0.auth0_integration_installed import Auth0IntegrationInstalled as Auth0IntegrationInstalled
from pypanther.rules.auth0.auth0_mfa_factor_setting_enabled import (
    Auth0MFAFactorSettingEnabled as Auth0MFAFactorSettingEnabled,
)
from pypanther.rules.auth0.auth0_mfa_policy_disabled import Auth0MFAPolicyDisabled as Auth0MFAPolicyDisabled
from pypanther.rules.auth0.auth0_mfa_policy_enabled import Auth0MFAPolicyEnabled as Auth0MFAPolicyEnabled
from pypanther.rules.auth0.auth0_mfa_risk_assessment_disabled import (
    Auth0MFARiskAssessmentDisabled as Auth0MFARiskAssessmentDisabled,
)
from pypanther.rules.auth0.auth0_mfa_risk_assessment_enabled import (
    Auth0MFARiskAssessmentEnabled as Auth0MFARiskAssessmentEnabled,
)
from pypanther.rules.auth0.auth0_post_login_action_flow import Auth0PostLoginActionFlow as Auth0PostLoginActionFlow
from pypanther.rules.auth0.auth0_user_invitation_created import Auth0UserInvitationCreated as Auth0UserInvitationCreated
from pypanther.rules.auth0.auth0_user_joined_tenant import Auth0UserJoinedTenant as Auth0UserJoinedTenant



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import defang_ioc, is_base64


@panther_managed
class StandardDNSBase64(Rule):
    display_name = "DNS Base64 Encoded Query"
    default_description = (
        "Detects DNS queries with Base64 encoded subdomains, which could indicate an attempt to obfuscate data exfil."
    )
    id = "Standard.DNSBase64-prototype"
    enabled = False
    default_reference = "https://zofixer.com/what-is-base64-disclosure-vulnerability/"
    default_severity = Severity.MEDIUM
    log_types = [LogType.CROWDSTRIKE_FDR_EVENT, LogType.AWS_VPC_DNS, LogType.CISCO_UMBRELLA_DNS]
    DECODED = ""

    def rule(self, event):
        query = event.udm("dns_query", default="")
        if not query:
            return False
        args = query.split(".")
        # Check if Base64 encoded arguments are present in the command line
        for arg in args:
            self.DECODED = is_base64(arg)
            if self.DECODED:
                return True
        return False

    def title(self, event):
        defang_query = defang_ioc(event.udm("dns_query")) if event.udm("dns_query") else "no query"
        return f"Base64 encoded query detected from [{event.udm('source_ip')}], [{defang_query}]"

    def alert_context(self, event):
        context = {}
        context["source ip"] = event.udm("source_ip")
        context["defanged query"] = defang_ioc(event.udm("dns_query")) if event.udm("dns_query") else "no query"
        context["decoded url part"] = self.DECODED
        return context

    tests = [
        RuleTest(
            name="AWS VPC DNS (Positive)",
            expected_result=True,
            log={
                "account_id": "123456789012",
                "answers": [{"Class": "IN", "Rdata": "172.31.46.187", "Type": "A"}],
                "p_log_type": "AWS.VPCDns",
                "query_class": "IN",
                "query_name": "c29tZSBsb25nIGJhc2U2NCBzdHJpbmc=.file1.16s.us",
                "query_timestamp": "2023-04-11 01:29:20",
                "query_type": "A",
                "rcode": "NOERROR",
                "region": "us-west-2",
                "srcaddr": "172.31.46.187",
                "srcids": {"instance": "i-09d9aa4e31675db61"},
                "srcport": "36899",
                "transport": "UDP",
                "version": "1.100000",
                "vpc_id": "vpc-c26c48ba",
            },
        ),
        RuleTest(
            name="AWS VPS DNS (Negative)",
            expected_result=False,
            log={
                "account_id": "123456789012",
                "answers": [{"Class": "IN", "Rdata": "172.31.46.187", "Type": "A"}],
                "p_log_type": "AWS.VPCDns",
                "query_class": "IN",
                "query_name": "test.com",
                "query_timestamp": "2023-04-11 01:29:20",
                "query_type": "A",
                "rcode": "NOERROR",
                "region": "us-west-2",
                "srcaddr": "172.31.46.187",
                "srcids": {"instance": "i-09d9aa4e31675db61"},
                "srcport": "36899",
                "transport": "UDP",
                "version": "1.100000",
                "vpc_id": "vpc-c26c48ba",
            },
        ),
        RuleTest(
            name="AWS VPS DNS subdomain (Negative)",
            expected_result=False,
            log={
                "account_id": "123456789012",
                "answers": [{"Class": "IN", "Rdata": "172.31.46.187", "Type": "A"}],
                "p_log_type": "AWS.VPCDns",
                "query_class": "IN",
                "query_name": "=.test.com",
                "query_timestamp": "2023-04-11 01:29:20",
                "query_type": "A",
                "rcode": "NOERROR",
                "region": "us-west-2",
                "srcaddr": "172.31.46.187",
                "srcids": {"instance": "i-09d9aa4e31675db61"},
                "srcport": "36899",
                "transport": "UDP",
                "version": "1.100000",
                "vpc_id": "vpc-c26c48ba",
            },
        ),
        RuleTest(
            name="Crowdstrike DNS Request (Positive)",
            expected_result=True,
            log={
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "1331552299",
                "ContextProcessId": "21866918",
                "ContextThreadId": "43270698197",
                "ContextTimeStamp": "2023-04-23 18:50:03.172",
                "Entitlements": "15",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event": {
                    "ConfigBuild": "1007.3.0016606.11",
                    "ConfigStateHash": "1331552299",
                    "ContextProcessId": "21866918",
                    "ContextThreadId": "43270698197",
                    "ContextTimeStamp": "1682275803.172",
                    "DnsRequestCount": "1",
                    "DomainName": "aGVsbG93b3JsZA==.example.com.",
                    "DualRequest": "0",
                    "EffectiveTransmissionClass": "3",
                    "Entitlements": "15",
                    "EventOrigin": "1",
                    "InterfaceIndex": "0",
                    "QueryStatus": "9003",
                    "RequestType": "1",
                    "aid": "877761efa8db44d792ddc2redacted",
                    "aip": "1.1.1.1",
                    "cid": "cfe698690964434083fecdredacted",
                    "event_platform": "Win",
                    "event_simpleName": "DnsRequest",
                    "id": "4f006a14-0fdf-474e-bfca-021a00a935ad",
                    "name": "DnsRequestV4",
                    "timestamp": "1682275805712",
                },
                "event_platform": "Win",
                "event_simpleName": "DnsRequest",
                "fdr_event_type": "DnsRequest",
                "id": "4f006a14-0fdf-474e-bfca-021a00a935ad",
                "name": "DnsRequestV4",
                "p_any_domain_names": ["win8.ipv6.microsoft.com"],
                "p_any_ip_addresses": ["1.1.1.1"],
                "p_any_md5_hashes": ["877761efa8db44d792ddc2redacted", "cfe698690964434083fecdredacted"],
                "p_any_trace_ids": ["877761efa8db44d792ddc2redacted", "cfe698690964434083fecdredacted"],
                "p_event_time": "2023-04-23 18:50:03.172",
                "p_log_type": "Crowdstrike.FDREvent",
                "p_parse_time": "2023-04-23 19:00:53.11",
                "p_row_id": "f2c89ec8f09cbc8ca2c1cbdf171a",
                "p_schema_version": 0,
                "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
                "p_source_label": "Crowdstrike-FDR-Dev",
                "timestamp": "2023-04-23 18:50:05.712",
            },
        ),
        RuleTest(
            name="Crowdstrike DNS Request (Negative)",
            expected_result=False,
            log={
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "1331552299",
                "ContextProcessId": "21866918",
                "ContextThreadId": "43270698197",
                "ContextTimeStamp": "2023-04-23 18:50:03.172",
                "Entitlements": "15",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event": {
                    "ConfigBuild": "1007.3.0016606.11",
                    "ConfigStateHash": "1331552299",
                    "ContextProcessId": "21866918",
                    "ContextThreadId": "43270698197",
                    "ContextTimeStamp": "1682275803.172",
                    "DnsRequestCount": "1",
                    "DomainName": "win8.ipv6.microsoft.com.",
                    "DualRequest": "0",
                    "EffectiveTransmissionClass": "3",
                    "Entitlements": "15",
                    "EventOrigin": "1",
                    "InterfaceIndex": "0",
                    "QueryStatus": "9003",
                    "RequestType": "1",
                    "aid": "877761efa8db44d792ddc2redacted",
                    "aip": "1.1.1.1",
                    "cid": "cfe698690964434083fecdredacted",
                    "event_platform": "Win",
                    "event_simpleName": "DnsRequest",
                    "id": "4f006a14-0fdf-474e-bfca-021a00a935ad",
                    "name": "DnsRequestV4",
                    "timestamp": "1682275805712",
                },
                "event_platform": "Win",
                "event_simpleName": "DnsRequest",
                "fdr_event_type": "DnsRequest",
                "id": "4f006a14-0fdf-474e-bfca-021a00a935ad",
                "name": "DnsRequestV4",
                "p_any_domain_names": ["win8.ipv6.microsoft.com"],
                "p_any_ip_addresses": ["1.1.1.1"],
                "p_any_md5_hashes": ["877761efa8db44d792ddc2redacted", "cfe698690964434083fecdredacted"],
                "p_any_trace_ids": ["877761efa8db44d792ddc2redacted", "cfe698690964434083fecdredacted"],
                "p_event_time": "2023-04-23 18:50:03.172",
                "p_log_type": "Crowdstrike.FDREvent",
                "p_parse_time": "2023-04-23 19:00:53.11",
                "p_row_id": "f2c89ec8f09cbc8ca2c1cbdf171a",
                "p_schema_version": 0,
                "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
                "p_source_label": "Crowdstrike-FDR-Dev",
                "timestamp": "2023-04-23 18:50:05.712",
            },
        ),
        RuleTest(
            name="Cisco Umbrella DNS Request (Positive)",
            expected_result=True,
            log={
                "action": "Allow",
                "internalIp": "136.24.229.58",
                "externalIp": "136.24.229.58",
                "timestamp": "2020-05-21 19:20:25.000",
                "responseCode": "NOERROR",
                "domain": "c29tZSBsb25nIGJhc2U2NCBzdHJpbmc=.example.io.",
                "p_log_type": "CiscoUmbrella.DNS",
            },
        ),
        RuleTest(
            name="Cisco Umbrella DNS Request (Negative)",
            expected_result=False,
            log={
                "action": "Allow",
                "internalIp": "136.24.229.58",
                "externalIp": "136.24.229.58",
                "timestamp": "2020-05-21 19:20:25.000",
                "responseCode": "NOERROR",
                "domain": "c29tZSBsb25IGJhc2.example.io.",
                "p_log_type": "CiscoUmbrella.DNS",
            },
        ),
        RuleTest(
            name="Crowdstrike no query",
            expected_result=False,
            log={
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "1331552299",
                "ContextProcessId": "21866918",
                "ContextThreadId": "43270698197",
                "ContextTimeStamp": "2023-04-23 18:50:03.172",
                "Entitlements": "15",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event": {
                    "ConfigBuild": "1007.3.0016606.11",
                    "ConfigStateHash": "1331552299",
                    "ContextProcessId": "21866918",
                    "ContextThreadId": "43270698197",
                    "ContextTimeStamp": "1682275803.172",
                    "DnsRequestCount": "1",
                    "DualRequest": "0",
                    "EffectiveTransmissionClass": "3",
                    "Entitlements": "15",
                    "EventOrigin": "1",
                    "InterfaceIndex": "0",
                    "QueryStatus": "9003",
                    "RequestType": "1",
                    "aid": "877761efa8db44d792ddc2redacted",
                    "aip": "1.1.1.1",
                    "cid": "cfe698690964434083fecdredacted",
                    "event_platform": "Win",
                    "event_simpleName": "DnsRequest",
                    "id": "4f006a14-0fdf-474e-bfca-021a00a935ad",
                    "name": "DnsRequestV4",
                    "timestamp": "1682275805712",
                },
                "event_platform": "Win",
                "event_simpleName": "DnsRequest",
                "fdr_event_type": "DnsRequest",
                "id": "4f006a14-0fdf-474e-bfca-021a00a935ad",
                "name": "DnsRequestV4",
                "p_any_domain_names": ["win8.ipv6.microsoft.com"],
                "p_any_ip_addresses": ["1.1.1.1"],
                "p_any_md5_hashes": ["877761efa8db44d792ddc2redacted", "cfe698690964434083fecdredacted"],
                "p_any_trace_ids": ["877761efa8db44d792ddc2redacted", "cfe698690964434083fecdredacted"],
                "p_event_time": "2023-04-23 18:50:03.172",
                "p_log_type": "Crowdstrike.FDREvent",
                "p_parse_time": "2023-04-23 19:00:53.11",
                "p_row_id": "f2c89ec8f09cbc8ca2c1cbdf171a",
                "p_schema_version": 0,
                "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
                "p_source_label": "Crowdstrike-FDR-Dev",
                "timestamp": "2023-04-23 18:50:05.712",
            },
        ),
    ]



import json

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import deep_get
from pypanther.helpers.okta import okta_alert_context


@panther_managed
class OktaNewBehaviorAccessingAdminConsole(Rule):
    id = "Okta.New.Behavior.Accessing.Admin.Console-prototype"
    display_name = "Okta New Behaviors Acessing Admin Console"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    reports = {"MITRE ATT&CK": ["TA0001:T1078.004"]}
    default_severity = Severity.HIGH
    default_description = "New Behaviors Observed while Accessing Okta Admin Console. A user attempted to access the Okta Admin Console from a new device with a new IP.\n"
    default_runbook = "Configure Authentication Policies (Application Sign-on Policies) for access to privileged applications, including the Admin Console, to require re-authentication “at every sign-in”. Turn on and test New Device and Suspicious Activity end-user notifications.\n"
    default_reference = "https://sec.okta.com/articles/2023/08/cross-tenant-impersonation-prevention-and-detection\n"

    def rule(self, event):
        if event.get("eventtype") != "policy.evaluate_sign_on":
            return False
        if "Okta Admin Console" not in event.deep_walk("target", "displayName", default=""):
            return False
        behaviors = event.deep_get("debugContext", "debugData", "behaviors")
        if behaviors:
            return "New Device=POSITIVE" in behaviors and "New IP=POSITIVE" in behaviors
        log_only_security_data = event.deep_get("debugContext", "debugData", "logOnlySecurityData")
        if isinstance(log_only_security_data, str):
            log_only_security_data = json.loads(log_only_security_data)
        return (
            deep_get(log_only_security_data, "behaviors", "New Device") == "POSITIVE"
            and deep_get(log_only_security_data, "behaviors", "New IP") == "POSITIVE"
        )

    def title(self, event):
        return f"{event.deep_get('actor', 'displayName', default='<displayName-not-found>')} <{event.deep_get('actor', 'alternateId', default='alternateId-not-found')}> accessed Okta Admin Console using new behaviors: New IP: {event.deep_get('client', 'ipAddress', default='<ipAddress-not-found>')} New Device: {event.deep_get('device', 'name', default='<deviceName-not-found>')}"

    def alert_context(self, event):
        return okta_alert_context(event)

    tests = [
        RuleTest(
            name="New Behavior Accessing Admin Console (behavior)",
            expected_result=True,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 20, "lon": -25},
                        "postalCode": "12345",
                        "state": "Ohio",
                    },
                    "ipAddress": "1.3.2.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML",
                        "like Gecko) Chrome/102.0.0.0 Safari/537.36": None,
                    },
                    "zone": "null",
                },
                "device": {"name": "Evil Computer"},
                "debugcontext": {
                    "debugData": {
                        "requestId": "AbCdEf12G",
                        "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                        "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                        "behaviors": [
                            "New Geo-Location=NEGATIVE",
                            "New Device=POSITIVE",
                            "New IP=POSITIVE",
                            "New State=NEGATIVE",
                            "New Country=NEGATIVE",
                            "Velocity=NEGATIVE",
                            "New City=NEGATIVE",
                        ],
                    },
                },
                "displaymessage": "Evaluation of sign-on policy",
                "eventtype": "policy.evaluate_sign_on",
                "outcome": {"reason": "Sign-on policy evaluation resulted in CHALLENGE", "result": "CHALLENGE"},
                "published": "2022-06-22 18:18:29.015",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Springfield",
                                "country": "United States",
                                "geolocation": {"lat": 20, "lon": -25},
                                "postalCode": "12345",
                                "state": "Ohio",
                                "ip": "1.3.2.4",
                                "version": "V4",
                            },
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 701,
                    "asOrg": "verizon",
                    "domain": "verizon.net",
                    "isProxy": False,
                    "isp": "verizon",
                },
                "severity": "INFO",
                "target": [
                    {"alternateId": "Okta Admin Console", "displayName": "Okta Admin Console", "type": "AppInstance"},
                    {
                        "alternateId": "peter.griffin@company.com",
                        "displayName": "Peter Griffin",
                        "id": "0002222AAAA",
                        "type": "User",
                    },
                ],
                "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
                "uuid": "AbC-123-XyZ",
                "version": "0",
            },
        ),
        RuleTest(
            name="New Behavior Accessing Admin Console (logSecurityDataOnly)",
            expected_result=True,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 20, "lon": -25},
                        "postalCode": "12345",
                        "state": "Ohio",
                    },
                    "ipAddress": "1.3.2.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML",
                        "like Gecko) Chrome/102.0.0.0 Safari/537.36": None,
                    },
                    "zone": "null",
                },
                "device": {"name": "Evil Computer"},
                "debugcontext": {
                    "debugData": {
                        "requestId": "AbCdEf12G",
                        "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                        "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                        "logOnlySecurityData": {
                            "risk": {"level": "LOW"},
                            "behaviors": {
                                "New Geo-Location": "NEGATIVE",
                                "New Device": "POSITIVE",
                                "New IP": "POSITIVE",
                                "New State": "NEGATIVE",
                                "New Country": "NEGATIVE",
                                "Velocity": "NEGATIVE",
                                "New City": "NEGATIVE",
                            },
                        },
                    },
                },
                "displaymessage": "Evaluation of sign-on policy",
                "eventtype": "policy.evaluate_sign_on",
                "outcome": {"reason": "Sign-on policy evaluation resulted in CHALLENGE", "result": "CHALLENGE"},
                "published": "2022-06-22 18:18:29.015",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Springfield",
                                "country": "United States",
                                "geolocation": {"lat": 20, "lon": -25},
                                "postalCode": "12345",
                                "state": "Ohio",
                                "ip": "1.3.2.4",
                                "version": "V4",
                            },
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 701,
                    "asOrg": "verizon",
                    "domain": "verizon.net",
                    "isProxy": False,
                    "isp": "verizon",
                },
                "severity": "INFO",
                "target": [
                    {"alternateId": "Okta Admin Console", "displayName": "Okta Admin Console", "type": "AppInstance"},
                    {
                        "alternateId": "peter.griffin@company.com",
                        "displayName": "Peter Griffin",
                        "id": "0002222AAAA",
                        "type": "User",
                    },
                ],
                "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
                "uuid": "AbC-123-XyZ",
                "version": "0",
            },
        ),
        RuleTest(
            name="Not New Behavior",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 20, "lon": -25},
                        "postalCode": "12345",
                        "state": "Ohio",
                    },
                    "ipAddress": "1.3.2.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML",
                        "like Gecko) Chrome/102.0.0.0 Safari/537.36": None,
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "requestId": "AbCdEf12G",
                        "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                        "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                        "logOnlySecurityData": {
                            "risk": {"level": "LOW"},
                            "behaviors": {
                                "New Geo-Location": "NEGATIVE",
                                "New Device": "NEGATIVE",
                                "New IP": "NEGATIVE",
                                "New State": "NEGATIVE",
                                "New Country": "NEGATIVE",
                                "Velocity": "NEGATIVE",
                                "New City": "NEGATIVE",
                            },
                        },
                    },
                },
                "displaymessage": "Evaluation of sign-on policy",
                "eventtype": "policy.evaluate_sign_on",
                "outcome": {"reason": "Sign-on policy evaluation resulted in CHALLENGE", "result": "CHALLENGE"},
                "published": "2022-06-22 18:18:29.015",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Springfield",
                                "country": "United States",
                                "geolocation": {"lat": 20, "lon": -25},
                                "postalCode": "12345",
                                "state": "Ohio",
                                "ip": "1.3.2.4",
                                "version": "V4",
                            },
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 701,
                    "asOrg": "verizon",
                    "domain": "verizon.net",
                    "isProxy": False,
                    "isp": "verizon",
                },
                "severity": "INFO",
                "target": [
                    {"alternateId": "Okta Admin Console", "displayName": "Okta Admin Console", "type": "AppInstance"},
                    {
                        "alternateId": "peter.griffin@company.com",
                        "displayName": "Peter Griffin",
                        "id": "0002222AAAA",
                        "type": "User",
                    },
                ],
                "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
                "uuid": "AbC-123-XyZ",
                "version": "0",
            },
        ),
        RuleTest(
            name="New Behavior Accessing Admin Console (logSecurityDataOnly) - not jsonified string",
            expected_result=True,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 20, "lon": -25},
                        "postalCode": "12345",
                        "state": "Ohio",
                    },
                    "ipAddress": "1.3.2.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML",
                        "like Gecko) Chrome/102.0.0.0 Safari/537.36": None,
                    },
                    "zone": "null",
                },
                "device": {"name": "Evil Computer"},
                "debugcontext": {
                    "debugData": {
                        "requestId": "AbCdEf12G",
                        "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                        "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                        "logOnlySecurityData": '{"risk":{"level":"LOW"},"behaviors":{"New Geo-Location":"NEGATIVE","New Device":"POSITIVE","New IP":"POSITIVE","New State":"NEGATIVE","New Country":"NEGATIVE","Velocity":"NEGATIVE","New City":"NEGATIVE"}}',
                    },
                },
                "displaymessage": "Evaluation of sign-on policy",
                "eventtype": "policy.evaluate_sign_on",
                "outcome": {"reason": "Sign-on policy evaluation resulted in CHALLENGE", "result": "CHALLENGE"},
                "published": "2022-06-22 18:18:29.015",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Springfield",
                                "country": "United States",
                                "geolocation": {"lat": 20, "lon": -25},
                                "postalCode": "12345",
                                "state": "Ohio",
                                "ip": "1.3.2.4",
                                "version": "V4",
                            },
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 701,
                    "asOrg": "verizon",
                    "domain": "verizon.net",
                    "isProxy": False,
                    "isp": "verizon",
                },
                "severity": "INFO",
                "target": [
                    {"alternateId": "Okta Admin Console", "displayName": "Okta Admin Console", "type": "AppInstance"},
                    {
                        "alternateId": "peter.griffin@company.com",
                        "displayName": "Peter Griffin",
                        "id": "0002222AAAA",
                        "type": "User",
                    },
                ],
                "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
                "uuid": "AbC-123-XyZ",
                "version": "0",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.slack import slack_alert_context


@panther_managed
class SlackAuditLogsLegalHoldPolicyModified(Rule):
    id = "Slack.AuditLogs.LegalHoldPolicyModified-prototype"
    display_name = "Slack Legal Hold Policy Modified"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Defense Evasion", "Impair Defenses", "Disable or Modify Tools"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562.001"]}
    default_severity = Severity.HIGH
    default_description = "Detects changes to configured legal hold policies"
    default_reference = "https://slack.com/intl/en-gb/help/articles/4401830811795-Create-and-manage-legal-holds"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    LEGAL_HOLD_POLICY_ACTIONS = {
        "legal_hold_policy_entities_deleted": "Slack Legal Hold Policy Entities Deleted",
        "legal_hold_policy_exclusion_added": "Slack Exclusions Added to Legal Hold Policy",
        "legal_hold_policy_released": "Slack Legal Hold Released",
        "legal_hold_policy_updated": "Slack Legal Hold Updated",
    }

    def rule(self, event):
        return event.get("action") in self.LEGAL_HOLD_POLICY_ACTIONS

    def title(self, event):
        # Only the `legal_hold_policy_updated` event includes relevant data to deduplicate
        if event.get("action") == "legal_hold_policy_updated":
            return f"Slack Legal Hold Updated [{event.deep_get('details', 'old_legal_hold_policy', 'name')}]"
        if event.get("action") in self.LEGAL_HOLD_POLICY_ACTIONS:
            return self.LEGAL_HOLD_POLICY_ACTIONS.get(event.get("action"))
        return "Slack Legal Hold Policy Modified"

    def alert_context(self, event):
        return slack_alert_context(event)

    tests = [
        RuleTest(
            name="Legal Hold - Entities Deleted",
            expected_result=True,
            log={
                "action": "legal_hold_policy_entities_deleted",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "A012B3CDEFG",
                        "name": "username",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace",
                        "id": "T01234N56GB",
                        "name": "test-workspace",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
            },
        ),
        RuleTest(
            name="Legal Hold - Exclusions Added",
            expected_result=True,
            log={
                "action": "legal_hold_policy_exclusion_added",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "A012B3CDEFG",
                        "name": "username",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace",
                        "id": "T01234N56GB",
                        "name": "test-workspace",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
            },
        ),
        RuleTest(
            name="Legal Hold - Policy Released",
            expected_result=True,
            log={
                "action": "legal_hold_policy_released",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "A012B3CDEFG",
                        "name": "username",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace",
                        "id": "T01234N56GB",
                        "name": "test-workspace",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
            },
        ),
        RuleTest(
            name="Legal Hold - Policy Updated",
            expected_result=True,
            log={
                "action": "legal_hold_policy_updated",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "A012B3CDEFG",
                        "name": "username",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace",
                        "id": "T01234N56GB",
                        "name": "test-workspace",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
            },
        ),
        RuleTest(
            name="User Logout",
            expected_result=False,
            log={
                "action": "user_logout",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace-1",
                        "id": "T01234N56GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
                "date_create": "2022-07-28 15:22:32",
                "entity": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
            },
        ),
    ]



from pypanther.rules.box.box_access_granted import BoxAccessGranted as BoxAccessGranted
from pypanther.rules.box.box_anomalous_download import BoxShieldAnomalousDownload as BoxShieldAnomalousDownload
from pypanther.rules.box.box_event_triggered_externally import (
    BoxEventTriggeredExternally as BoxEventTriggeredExternally,
)
from pypanther.rules.box.box_item_shared_externally import BoxItemSharedExternally as BoxItemSharedExternally
from pypanther.rules.box.box_malicious_content import BoxMaliciousContent as BoxMaliciousContent
from pypanther.rules.box.box_new_login import BoxNewLogin as BoxNewLogin
from pypanther.rules.box.box_policy_violation import (
    BoxContentWorkflowPolicyViolation as BoxContentWorkflowPolicyViolation,
)
from pypanther.rules.box.box_suspicious_login_or_session import BoxShieldSuspiciousAlert as BoxShieldSuspiciousAlert
from pypanther.rules.box.box_untrusted_device import BoxUntrustedDevice as BoxUntrustedDevice
from pypanther.rules.box.box_user_downloads import BoxLargeNumberDownloads as BoxLargeNumberDownloads
from pypanther.rules.box.box_user_permission_updates import (
    BoxLargeNumberPermissionUpdates as BoxLargeNumberPermissionUpdates,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteLoginType(Rule):
    id = "GSuite.LoginType-prototype"
    display_name = "GSuite Login Type"
    enabled = False
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Configuration Required", "Initial Access:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    default_severity = Severity.MEDIUM
    default_description = "A login of a non-approved type was detected for this user.\n"
    default_reference = "https://support.google.com/a/answer/9039184?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Correct the user account settings so that only logins of approved types are available.\n"
    summary_attributes = ["actor:email"]
    # allow-list of approved login types
    APPROVED_LOGIN_TYPES = {"exchange", "google_password", "reauth", "saml", "unknown"}
    # allow-list any application names here
    APPROVED_APPLICATION_NAMES = {"saml"}

    def rule(self, event):
        if event.get("type") != "login":
            return False
        if event.get("name") == "logout":
            return False
        if (
            event.deep_get("parameters", "login_type") in self.APPROVED_LOGIN_TYPES
            or event.deep_get("id", "applicationName") in self.APPROVED_APPLICATION_NAMES
        ):
            return False
        return True

    def title(self, event):
        return f"A login attempt of a non-approved type was detected for user [{event.deep_get('actor', 'email', default='<UNKNOWN_USER>')}]"

    tests = [
        RuleTest(
            name="Login With Approved Type",
            expected_result=False,
            log={
                "id": {"applicationName": "login"},
                "actor": {"email": "some.user@somedomain.com"},
                "type": "login",
                "name": "login_success",
                "parameters": {"login_type": "saml"},
            },
        ),
        RuleTest(
            name="Login With Unapproved Type",
            expected_result=True,
            log={
                "id": {"applicationName": "login"},
                "actor": {"email": "some.user@somedomain.com"},
                "type": "login",
                "name": "login_success",
                "parameters": {"login_type": "turbo-snail"},
            },
        ),
        RuleTest(
            name="Non-Login event",
            expected_result=False,
            log={
                "id": {"applicationName": "logout"},
                "actor": {"email": "some.user@somedomain.com"},
                "type": "login",
                "name": "login_success",
                "parameters": {"login_type": "saml"},
            },
        ),
        RuleTest(
            name="Saml Login Event",
            expected_result=False,
            log={
                "actor": {"email": "some.user@somedomain.com"},
                "id": {"applicationName": "saml", "time": "2022-05-26 15:26:09.421000000"},
                "ipAddress": "10.10.10.10",
                "kind": "admin#reports#activity",
                "name": "login_success",
                "parameters": {
                    "application_name": "Some SAML Application",
                    "initiated_by": "sp",
                    "orgunit_path": "/SomeOrgUnit",
                    "saml_status_code": "SUCCESS_URI",
                },
                "type": "login",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class RoleAssumedbyUser(Rule):
    id = "Role.Assumed.by.User-prototype"
    display_name = "SIGNAL - Role Assumed by User"
    enabled = False
    create_alert = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO

    def rule(self, event):
        aws_service = event.deep_get("userIdentity", "type") == "AWSService"
        return all(
            [event.get("eventName") == "AssumeRole", event.deep_get("requestParameters", "roleArn"), not aws_service],
        )

    tests = [
        RuleTest(
            name="Role Assumed by Service",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "1f3d7d49-6637-3304-b959-9be15f20215d",
                "eventName": "AssumeRole",
                "eventSource": "sts.amazonaws.com",
                "eventTime": "2024-06-02 20:27:12",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "123456789123",
                "requestID": "a0dda101-6e27-4f88-8250-f3d475f88b56",
                "requestParameters": {
                    "roleArn": "arn:aws:iam::123456789123:role/my_role_arn",
                    "roleSessionName": "awslambda_55_20240602202712548",
                },
                "resources": [
                    {
                        "accountId": "123456789123",
                        "arn": "arn:aws:iam::123456789123:role/my_role_arn",
                        "type": "AWS::IAM::Role",
                    },
                ],
                "responseElements": {
                    "credentials": {
                        "accessKeyId": "REDACTED",
                        "expiration": "Jun 2, 2024, 10:37:12 PM",
                        "sessionToken": "REDACTED",
                    },
                },
                "sharedEventID": "95e84e79-100a-40a6-985e-3c9c4b41f622",
                "sourceIPAddress": "lambda.amazonaws.com",
                "userAgent": "lambda.amazonaws.com",
                "userIdentity": {"invokedBy": "lambda.amazonaws.com", "type": "AWSService"},
            },
        ),
    ]



from pypanther.rules.gcp_audit.gcp_access_attempts_violating_vpc_service_controls import (
    GCPAccessAttemptsViolatingVPCServiceControls as GCPAccessAttemptsViolatingVPCServiceControls,
)
from pypanther.rules.gcp_audit.gcp_bigquery_large_scan import GCPBigQueryLargeScan as GCPBigQueryLargeScan
from pypanther.rules.gcp_audit.gcp_cloud_run_service_created import (
    GCPCloudRunServiceCreated as GCPCloudRunServiceCreated,
)
from pypanther.rules.gcp_audit.gcp_cloud_run_set_iam_policy import GCPCloudRunSetIAMPolicy as GCPCloudRunSetIAMPolicy
from pypanther.rules.gcp_audit.gcp_cloud_storage_buckets_modified_or_deleted import (
    GCPCloudStorageBucketsModifiedOrDeleted as GCPCloudStorageBucketsModifiedOrDeleted,
)
from pypanther.rules.gcp_audit.gcp_cloudbuild_potential_privilege_escalation import (
    GCPCloudBuildPotentialPrivilegeEscalation as GCPCloudBuildPotentialPrivilegeEscalation,
)
from pypanther.rules.gcp_audit.gcp_cloudfunctions_functions_create import (
    GCPCloudfunctionsFunctionsCreate as GCPCloudfunctionsFunctionsCreate,
)
from pypanther.rules.gcp_audit.gcp_cloudfunctions_functions_update import (
    GCPCloudfunctionsFunctionsUpdate as GCPCloudfunctionsFunctionsUpdate,
)
from pypanther.rules.gcp_audit.gcp_compute_set_iam_policy import GCPComputeIAMPolicyUpdate as GCPComputeIAMPolicyUpdate
from pypanther.rules.gcp_audit.gcp_computeinstances_create_privilege_escalation import (
    GCPcomputeinstancescreatePrivilegeEscalation as GCPcomputeinstancescreatePrivilegeEscalation,
)
from pypanther.rules.gcp_audit.gcp_destructive_queries import GCPDestructiveQueries as GCPDestructiveQueries
from pypanther.rules.gcp_audit.gcp_dns_zone_modified_or_deleted import (
    GCPDNSZoneModifiedorDeleted as GCPDNSZoneModifiedorDeleted,
)
from pypanther.rules.gcp_audit.gcp_firewall_rule_created import GCPFirewallRuleCreated as GCPFirewallRuleCreated
from pypanther.rules.gcp_audit.gcp_firewall_rule_deleted import GCPFirewallRuleDeleted as GCPFirewallRuleDeleted
from pypanther.rules.gcp_audit.gcp_firewall_rule_modified import GCPFirewallRuleModified as GCPFirewallRuleModified
from pypanther.rules.gcp_audit.gcp_gcs_iam_changes import GCPGCSIAMChanges as GCPGCSIAMChanges
from pypanther.rules.gcp_audit.gcp_gcs_public import GCPGCSPublic as GCPGCSPublic
from pypanther.rules.gcp_audit.gcp_iam_corp_email import GCPIAMCorporateEmail as GCPIAMCorporateEmail
from pypanther.rules.gcp_audit.gcp_iam_custom_role_changes import GCPIAMCustomRoleChanges as GCPIAMCustomRoleChanges
from pypanther.rules.gcp_audit.gcp_iam_org_folder_changes import GCPIAMOrgFolderIAMChanges as GCPIAMOrgFolderIAMChanges
from pypanther.rules.gcp_audit.gcp_iam_roles_update_privilege_escalation import (
    GCPiamrolesupdatePrivilegeEscalation as GCPiamrolesupdatePrivilegeEscalation,
)
from pypanther.rules.gcp_audit.gcp_iam_service_account_key_create import (
    GCPiamserviceAccountKeyscreate as GCPiamserviceAccountKeyscreate,
)
from pypanther.rules.gcp_audit.gcp_iam_service_accounts_get_access_token_privilege_escalation import (
    GCPIAMserviceAccountsgetAccessTokenPrivilegeEscalation as GCPIAMserviceAccountsgetAccessTokenPrivilegeEscalation,
)
from pypanther.rules.gcp_audit.gcp_iam_service_accounts_sign_blob import (
    GCPIAMserviceAccountssignBlob as GCPIAMserviceAccountssignBlob,
)
from pypanther.rules.gcp_audit.gcp_iam_serviceaccounts_signjwt import (
    GCPIAMserviceAccountssignJwtPrivilegeEscalation as GCPIAMserviceAccountssignJwtPrivilegeEscalation,
)
from pypanther.rules.gcp_audit.gcp_inbound_sso_profile_created_or_updated import (
    GCPInboundSSOProfileCreated as GCPInboundSSOProfileCreated,
)
from pypanther.rules.gcp_audit.gcp_invite_external_user_as_owner import (
    GCPProjectExternalUserOwnershipInvite as GCPProjectExternalUserOwnershipInvite,
)
from pypanther.rules.gcp_audit.gcp_log_bucket_or_sink_deleted import (
    GCPLogBucketOrSinkDeleted as GCPLogBucketOrSinkDeleted,
)
from pypanther.rules.gcp_audit.gcp_logging_settings_modified import (
    GCPLoggingSettingsModified as GCPLoggingSettingsModified,
)
from pypanther.rules.gcp_audit.gcp_logging_sink_modified import GCPLoggingSinkModified as GCPLoggingSinkModified
from pypanther.rules.gcp_audit.gcp_permissions_granted_to_create_or_manage_service_account_key import (
    GCPPermissionsGrantedtoCreateorManageServiceAccountKey as GCPPermissionsGrantedtoCreateorManageServiceAccountKey,
)
from pypanther.rules.gcp_audit.gcp_privilege_escalation_by_deployments_create import (
    GCPPrivilegeEscalationByDeploymentsCreate as GCPPrivilegeEscalationByDeploymentsCreate,
)
from pypanther.rules.gcp_audit.gcp_service_account_access_denied import (
    GCPServiceAccountAccessDenied as GCPServiceAccountAccessDenied,
)
from pypanther.rules.gcp_audit.gcp_service_account_or_keys_created import (
    GCPServiceAccountorKeysCreated as GCPServiceAccountorKeysCreated,
)
from pypanther.rules.gcp_audit.gcp_serviceusage_apikeys_create_privilege_escalation import (
    GCPserviceusageapiKeyscreatePrivilegeEscalation as GCPserviceusageapiKeyscreatePrivilegeEscalation,
)
from pypanther.rules.gcp_audit.gcp_snapshot_insert import (
    GCPComputeSnapshotUnexpectedDomain as GCPComputeSnapshotUnexpectedDomain,
)
from pypanther.rules.gcp_audit.gcp_sql_config_changes import GCPSQLConfigChanges as GCPSQLConfigChanges
from pypanther.rules.gcp_audit.gcp_storage_hmac_keys_create import GCPStorageHmacKeysCreate as GCPStorageHmacKeysCreate
from pypanther.rules.gcp_audit.gcp_unused_regions import GCPUnusedRegions as GCPUnusedRegions
from pypanther.rules.gcp_audit.gcp_user_added_to_iap_protected_service import (
    GCPUserAddedtoIAPProtectedService as GCPUserAddedtoIAPProtectedService,
)
from pypanther.rules.gcp_audit.gcp_user_added_to_privileged_group import (
    GCPUserAddedToPrivilegedGroup as GCPUserAddedToPrivilegedGroup,
)
from pypanther.rules.gcp_audit.gcp_vpc_flow_logs_disabled import GCPVPCFlowLogsDisabled as GCPVPCFlowLogsDisabled
from pypanther.rules.gcp_audit.gcp_workforce_pool_created_or_updated import (
    GCPWorkforcePoolCreatedorUpdated as GCPWorkforcePoolCreatedorUpdated,
)
from pypanther.rules.gcp_audit.gcp_workload_identity_pool_created_or_updated import (
    GCPWorkloadIdentityPoolCreatedorUpdated as GCPWorkloadIdentityPoolCreatedorUpdated,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers import event_type
from pypanther.helpers.zendesk import zendesk_get_roles


@panther_managed
class ZendeskUserRoleChanged(Rule):
    id = "Zendesk.UserRoleChanged-prototype"
    display_name = "Zendesk User Role Changed"
    log_types = [LogType.ZENDESK_AUDIT]
    default_severity = Severity.INFO
    default_description = "A user's Zendesk role was changed"
    default_reference = (
        "https://support.zendesk.com/hc/en-us/articles/4408824375450-Setting-roles-and-access-in-Zendesk-Admin-Center"
    )
    summary_attributes = ["p_any_ip_addresses"]

    def rule(self, event):
        if event.get("source_type") == "user" and event.get("action") == "update":
            # admin roles have their own handling
            if (
                event.udm("event_type") != event_type.ADMIN_ROLE_ASSIGNED
                and "role changed" in event.get("change_description", "").lower()
            ):
                _, new_role = zendesk_get_roles(event)
                return bool(new_role)
        return False

    def title(self, event):
        old_role, new_role = zendesk_get_roles(event)
        return (
            f"Actor user [{event.udm('actor_user')}] changed [{event.udm('user')}] role from {old_role} to {new_role}"
        )

    tests = [
        RuleTest(
            name="Zendesk - Role Changed",
            expected_result=True,
            log={
                "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
                "id": 123456789123,
                "action_label": "Updated",
                "actor_id": 123,
                "actor_name": "John Doe",
                "source_id": 123,
                "source_type": "user",
                "source_label": "Bob Cat",
                "action": "update",
                "change_description": "Role changed from Administrator to End User",
                "ip_address": "127.0.0.1",
                "created_at": "2021-05-28T18:39:50Z",
                "p_log_type": "Zendesk.Audit",
            },
        ),
        RuleTest(
            name="Zendesk - Admin Role Assigned",
            expected_result=False,
            log={
                "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
                "id": 123456789123,
                "action_label": "Updated",
                "actor_id": 123,
                "actor_name": "John Doe",
                "source_id": 123,
                "source_type": "user",
                "source_label": "Bob Cat",
                "action": "update",
                "change_description": "Role changed from End User to Administrator",
                "ip_address": "127.0.0.1",
                "created_at": "2021-05-28T18:39:50Z",
                "p_log_type": "Zendesk.Audit",
            },
        ),
        RuleTest(
            name="Zendesk - No changing roles",
            expected_result=False,
            log={
                "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
                "id": 123456789123,
                "action_label": "Updated",
                "actor_id": 123,
                "actor_name": "John Doe",
                "source_id": 123,
                "source_type": "user",
                "source_label": "Bob Cat",
                "action": "update",
                "change_description": "Organization: AAAA is asigned",
                "ip_address": "127.0.0.1",
                "created_at": "2021-05-28T18:39:50Z",
                "p_log_type": "Zendesk.Audit",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.slack import slack_alert_context


@panther_managed
class SlackAuditLogsPassthroughAnomaly(Rule):
    id = "Slack.AuditLogs.PassthroughAnomaly-prototype"
    display_name = "Slack Anomaly Detected"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    default_severity = Severity.LOW
    reports = {"MITRE ATT&CK": ["TA0011:T1071"]}
    default_description = "Passthrough for anomalies detected by Slack"
    default_reference = "https://api.slack.com/admins/audit-logs-anomaly"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    tags = ["Slack", "Command and Control", "Application Layer Protocol"]
    ELEVATED_ANOMALIES = {"excessive_malware_uploads", "session_fingerprint", "unexpected_admin_action"}

    def rule(self, event):
        return event.get("action") == "anomaly"

    def severity(self, event):
        # Return "MEDIUM" for some more serious anomalies
        reasons = event.deep_get("details", "reason", default=[])
        if set(reasons) & self.ELEVATED_ANOMALIES:
            return "MEDIUM"
        return "DEFAULT"

    def alert_context(self, event):
        context = slack_alert_context(event)
        context |= {"details": event.get("details", {}), "context": event.get("context", {})}
        return context

    tests = [
        RuleTest(
            name="Name",
            expected_result=True,
            log={
                "action": "anomaly",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace-1",
                        "id": "T01234N56GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
            },
        ),
        RuleTest(
            name="User Logout",
            expected_result=False,
            log={
                "action": "user_logout",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace-1",
                        "id": "T01234N56GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
                "date_create": "2022-07-28 15:22:32",
                "entity": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
            },
        ),
        RuleTest(
            name="Session Fingerprint",
            expected_result=True,
            log={
                "action": "anomaly",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace-1",
                        "id": "T01234N56GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
                "date_create": "2024-08-19 13:54:53.000000000",
                "details": {
                    "action_timestamp": 1724075641026703,
                    "location": "London, UK",
                    "previous_ip_address": "",
                    "previous_ua": "",
                    "reason": ["session_fingerprint"],
                },
                "entity": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "id": "95edcb27-132a-4420-9229-783b38a16b5a",
                "p_event_time": "2024-08-19 13:54:53.000000000",
                "p_log_type": "Slack.AuditLogs",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context, lookup_aws_account_name


@panther_managed
class AWSVPCFlowLogsDeleted(Rule):
    id = "AWS.VPCFlow.LogsDeleted-prototype"
    display_name = "AWS VPC Flow Logs Removed"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.LOW
    reports = {"MITRE ATT&CK": ["TA0005:T1562.008"]}
    default_description = "Detects when logs for a VPC have been removed."
    default_reference = "https://stratus-red-team.cloud/attack-techniques/AWS/aws.defense-evasion.vpc-remove-flow-logs/"
    default_runbook = "Look for an accompanying 'DeleteVpc' event, and confirm that they are related. if there is no matching VPC Deletion event, followup with the log removal to determine if it is legitimate."
    tags = [
        "AWS",
        "Cloudtrail",
        "Defense Evasion",
        "Impair Defenses",
        "Disable or Modify Cloud Logs",
        "Defense Evasion:Impair Defenses",
        "Security Control",
    ]

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") == "DeleteFlowLogs"

    def title(self, event):
        account = event.deep_get("userIdentity", "accountId", default="<UNKNOWN ACCOUNT>")
        region = event.get("awsRegion", "<UNKNOWN REGION>")
        return f"VPC Flow logs have been deleted in {lookup_aws_account_name(account)} in {region}"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Logs Deleted",
            expected_result=True,
            log={
                "p_event_time": "2024-11-26 19:29:38.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-11-26 19:35:54.358700257",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "d5e6d49c-0be9-4c53-ab8a-c7ca86edd130",
                "eventName": "DeleteFlowLogs",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2024-11-26 19:29:38.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "1dcd7d36-72be-4aad-9e9d-93b88e0135ea",
                "requestParameters": {
                    "DeleteFlowLogsRequest": {"FlowLogId": {"content": "fl-0ef673ef70c4f07cc", "tag": 1}},
                },
                "responseElements": {
                    "DeleteFlowLogsResponse": {
                        "requestId": "1dcd7d36-72be-4aad-9e9d-93b88e0135ea",
                        "unsuccessful": "",
                        "xmlns": "http://ec2.amazonaws.com/doc/2016-11-15/",
                    },
                },
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "ec2.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "sample-user-agent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/leroy.jenkins",
                    "principalId": "SAMPLE_PRINCIPAL_ID:leroy.jenkins",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-11-26T17:05:50Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther.rules.cloudflare.cloudflare_firewall_ddos import CloudflareFirewallL7DDoS as CloudflareFirewallL7DDoS
from pypanther.rules.cloudflare.cloudflare_httpreq_bot_high_volume import (
    CloudflareHttpRequestBotHighVolume as CloudflareHttpRequestBotHighVolume,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GSuiteWorkspacePasswordReuseEnabled(Rule):
    id = "GSuite.Workspace.PasswordReuseEnabled-prototype"
    display_name = "GSuite Workspace Password Reuse Has Been Enabled"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.HIGH
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_description = "A Workspace Admin Has Enabled Password Reuse\n"
    default_reference = "https://support.google.com/a/answer/139399?hl=en#"
    default_runbook = "Verify the intent of this Password Reuse Setting Change. If intent cannot be verified, then a search on the actor's other activities is advised.\n"
    summary_attributes = ["actor:email"]

    def rule(self, event):
        if event.deep_get("id", "applicationName", default="").lower() != "admin":
            return False
        if all(
            [
                event.get("name", "") == "CHANGE_APPLICATION_SETTING",
                event.get("type", "") == "APPLICATION_SETTINGS",
                event.deep_get("parameters", "NEW_VALUE", default="").lower() == "true",
                event.deep_get("parameters", "SETTING_NAME", default="")
                == "Password Management - Enable password reuse",
            ],
        ):
            return True
        return False

    def title(self, event):
        return f"GSuite Workspace Password Reuse Has Been Enabled By [{event.deep_get('actor', 'email', default='<NO_ACTOR_FOUND>')}]"

    tests = [
        RuleTest(
            name="Workspace Admin Enabled Password Reuse",
            expected_result=True,
            log={
                "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
                "id": {
                    "applicationName": "admin",
                    "customerId": "D12345",
                    "time": "2022-12-11 01:18:47.973000000",
                    "uniqueQualifier": "-12345",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "CHANGE_APPLICATION_SETTING",
                "parameters": {
                    "APPLICATION_EDITION": "standard",
                    "APPLICATION_NAME": "Security",
                    "NEW_VALUE": "true",
                    "OLD_VALUE": "false",
                    "ORG_UNIT_NAME": "Example IO",
                    "SETTING_NAME": "Password Management - Enable password reuse",
                },
                "type": "APPLICATION_SETTINGS",
            },
        ),
        RuleTest(
            name="Admin Set Default Calendar SHARING_OUTSIDE_DOMAIN Setting to READ_ONLY_ACCESS",
            expected_result=False,
            log={
                "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
                "id": {
                    "applicationName": "admin",
                    "customerId": "D12345",
                    "time": "2022-12-11 01:06:26.303000000",
                    "uniqueQualifier": "-12345",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "CHANGE_CALENDAR_SETTING",
                "parameters": {
                    "DOMAIN_NAME": "example.io",
                    "NEW_VALUE": "READ_ONLY_ACCESS",
                    "OLD_VALUE": "DEFAULT",
                    "ORG_UNIT_NAME": "Example IO",
                    "SETTING_NAME": "SHARING_OUTSIDE_DOMAIN",
                },
                "type": "CALENDAR_SETTINGS",
            },
        ),
        RuleTest(
            name="ListObject Type",
            expected_result=False,
            log={
                "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
                "id": {
                    "applicationName": "drive",
                    "customerId": "D12345",
                    "time": "2022-12-20 17:27:47.080000000",
                    "uniqueQualifier": "-7312729053723258069",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "rename",
                "parameters": {
                    "actor_is_collaborator_account": None,
                    "billable": True,
                    "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                    "doc_title": "Document Title- Found Here",
                    "doc_type": "presentation",
                    "is_encrypted": None,
                    "new_value": ["Document Title- Found Here"],
                    "old_value": ["Document Title- Old"],
                    "owner": "user@example.io",
                    "owner_is_shared_drive": None,
                    "owner_is_team_drive": None,
                    "primary_event": True,
                    "visibility": "private",
                },
                "type": "access",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class OsquerySSHListener(Rule):
    id = "Osquery.SSHListener-prototype"
    display_name = "OSQuery Detected SSH Listener"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "Lateral Movement:Remote Services"]
    reports = {"MITRE ATT&CK": ["TA0008:T1021"]}
    default_severity = Severity.MEDIUM
    default_description = "Check if SSH is listening in a non-production environment. This could be an indicator of persistent access within an environment.\n"
    default_runbook = "Terminate the SSH daemon, investigate for signs of compromise.\n"
    default_reference = "https://medium.com/uptycs/osquery-what-it-is-how-it-works-and-how-to-use-it-ce4e81e60dfc"
    summary_attributes = ["action", "hostIdentifier", "name"]

    def rule(self, event):
        return (
            event.get("name") == "pack_incident-response_listening_ports"
            and event.deep_get("columns", "port") == "22"
            and (event.get("action") == "added")
        )

    tests = [
        RuleTest(
            name="SSH Listener Detected",
            expected_result=True,
            log={
                "action": "added",
                "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
                "columns": {
                    "build_distro": "10.12",
                    "build_platform": "darwin",
                    "config_hash": "1111",
                    "config_valid": "1",
                    "counter": "14",
                    "global_state": "0",
                    "extensions": "active",
                    "instance_id": "1111",
                    "pid": "223",
                    "port": "22",
                    "resident_size": "54894592",
                    "start_time": "1536634519",
                    "system_time": "12472",
                    "user_time": "31800",
                    "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                    "version": "Not Supported",
                    "watcher": "92",
                },
                "counter": "255",
                "decorations": {"host_uuid": "1111", "environment": "corp"},
                "epoch": "0",
                "hostIdentifier": "test.lan",
                "log_type": "result",
                "name": "pack_incident-response_listening_ports",
                "unixTime": "1536682461",
            },
        ),
        RuleTest(
            name="SSH Listener Not Detected",
            expected_result=False,
            log={
                "action": "added",
                "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
                "columns": {
                    "build_distro": "10.12",
                    "build_platform": "darwin",
                    "config_hash": "1111",
                    "config_valid": "1",
                    "counter": "14",
                    "global_state": "2",
                    "extensions": "active",
                    "instance_id": "1111",
                    "pid": "223",
                    "port": "443",
                    "resident_size": "54894592",
                    "start_time": "1536634519",
                    "system_time": "12472",
                    "user_time": "31800",
                    "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                    "version": "10.14.2",
                    "watcher": "92",
                },
                "counter": "255",
                "decorations": {"host_uuid": "1111", "environment": "corp"},
                "epoch": "0",
                "hostIdentifier": "test.lan",
                "log_type": "result",
                "name": "pack_incident-response_listening_ports",
                "unixTime": "1536682461",
            },
        ),
    ]



from panther_core import PantherEvent

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSCloudTrailSESCheckIdentityVerifications(Rule):
    id = "AWS.CloudTrail.SES.CheckIdentityVerifications-prototype"
    display_name = "AWS CloudTrail SES Check Identity Verifications"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    create_alert = False
    default_reference = "https://stratus-red-team.cloud/attack-techniques/AWS/aws.discovery.ses-enumerate/\n"
    tags = ["AWS CloudTrail"]

    def rule(self, event: PantherEvent) -> bool:
        return event.get("eventName") == "GetIdentityVerificationAttributes"

    def alert_context(self, event: PantherEvent) -> dict:
        context = aws_rule_context(event)
        context["accountRegion"] = f"{event.get('recipientAccountId')}_{event.get('eventRegion')}"
        return context

    tests = [
        RuleTest(
            name="GetIdentityVerificationStatus Event",
            expected_result=True,
            log={
                "p_event_time": "2025-01-20 16:52:14.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2025-01-20 17:00:54.142940079",
                "additionalEventData": {"SignatureVersion": "4"},
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "05197e93-992f-4476-899a-a6f53c9a462c",
                "eventName": "GetIdentityVerificationAttributes",
                "eventSource": "ses.amazonaws.com",
                "eventTime": "2025-01-20 16:52:14.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "111122223333",
                "requestID": "e3b6e034-97ce-4d43-a7d2-1e718f3ebf32",
                "requestParameters": {
                    "identities": ["acme.com", "bobson.dugnutt@acme.com", "sleve.mcdichael@yahoo.com"],
                },
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "email.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "example-user-agent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/bobson.dugnutt",
                    "principalId": "SAMPLE_PRINCIPAL_ID:bobson.dugnutt",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-01-20T15:58:59Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.notion import notion_alert_context


@panther_managed
class NotionAuditLogExported(Rule):
    id = "Notion.Audit.Log.Exported-prototype"
    display_name = "Notion Audit Log Exported"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Data Security", "Data Exfiltration"]
    default_severity = Severity.MEDIUM
    default_description = "A Notion User exported audit logs for your organization’s workspace."
    default_runbook = "Possible Data Exfiltration. Follow up with the Notion User to determine if this was done for a valid business reason."
    default_reference = "https://www.notion.so/help/audit-log#export-your-audit-log"

    def rule(self, event):
        event_type = event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>")
        return event_type == "workspace.audit_log_exported"

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        workspace_id = event.deep_get("event", "workspace_id", default="<NO_WORKSPACE_ID_FOUND>")
        duration_in_days = event.deep_get("event", "details", "duration_in_days", default="<NO_DURATION_IN_DAYS_FOUND>")
        return f"Notion User [{user}] exported audit logs for the last {duration_in_days} days for workspace id {workspace_id}"

    def alert_context(self, event):
        return notion_alert_context(event)

    tests = [
        RuleTest(
            name="Other Event",
            expected_result=False,
            log={
                "event": {
                    "id": "...",
                    "timestamp": "2023-05-15T19:14:21.031Z",
                    "workspace_id": "..",
                    "actor": {
                        "id": "..",
                        "object": "user",
                        "type": "person",
                        "person": {"email": "homer.simpson@yourcompany.io"},
                    },
                    "ip_address": "...",
                    "platform": "web",
                    "type": "workspace.content_exported",
                    "workspace.content_exported": {},
                },
            },
        ),
        RuleTest(
            name="Audit Log Exported",
            expected_result=True,
            log={
                "event": {
                    "id": "...",
                    "timestamp": "2023-05-15T19:14:21.031Z",
                    "workspace_id": "..",
                    "actor": {
                        "object": "user",
                        "id": "..",
                        "type": "person",
                        "person": {"email": "homer.simpson@yourcompany.io"},
                    },
                    "ip_address": "...",
                    "platform": "web",
                    "type": "workspace.audit_log_exported",
                    "details": {"duration_in_days": 30},
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.okta import okta_alert_context


@panther_managed
class OktaThreatInsightSecurityThreatDetected(Rule):
    default_description = "Okta ThreatInsight identified request from potentially malicious IP address"
    default_reference = (
        "https://help.okta.com/en-us/Content/Topics/Security/threat-insight/configure-threatinsight-system-log.htm"
    )
    display_name = "Okta ThreatInsight Security Threat Detected"
    default_severity = Severity.HIGH
    log_types = [LogType.OKTA_SYSTEM_LOG]
    id = "Okta.ThreatInsight.Security.Threat.Detected-prototype"

    def severity_from_threat_string(self, threat_detection):
        # threat detection is a string but contains json data
        # can contain multiple threats detected with multiple severities
        # return highest found severity
        if "CRITICAL" in threat_detection:
            return "CRITICAL"
        if "HIGH" in threat_detection:
            return "HIGH"
        if "MEDIUM" in threat_detection:
            return "MEDIUM"
        if "LOW" in threat_detection:
            return "LOW"
        if "INFO" in threat_detection:
            return "INFO"
        return "MEDIUM"

    def rule(self, event):
        return event.get("eventtype") == "security.threat.detected"

    def title(self, event):
        return f"Okta: ThreatInsight identified potentially malicious behavior for [{event.get('actor', {}).get('displayName', '<display-name-not-found>')}]"

    def severity(self, event):
        outcome = event.deep_get("outcome", "result", default="<OUTCOME_NOT_FOUND>")
        if outcome == "DENY":
            return "INFO"
        threat_detection = (
            event.get("debugcontext", {}).get("debugData", {}).get("threatDetections", "<threat-detection-not-found>")
        )
        return self.severity_from_threat_string(threat_detection)

    def alert_context(self, event):
        return okta_alert_context(event)

    tests = [
        RuleTest(
            name="Other Event",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc456",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abc12345"},
                "client": {
                    "device": "Unknown",
                    "ipAddress": "1.2.3.4",
                    "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Chrome"},
                    "zone": "null",
                },
                "debugcontext": {"debugData": {}},
                "eventtype": "application.integration.rate_limit_exceeded",
                "legacyeventtype": "app.api.error.rate.limit.exceeded",
                "outcome": {"result": "SUCCESS"},
                "published": "2022-06-10 17:19:58.423",
                "request": {},
                "securitycontext": {},
                "severity": "INFO",
                "target": [{"alternateId": "App ", "displayName": "App", "id": "12345", "type": "AppInstance"}],
                "transaction": {"detail": {}, "id": "sdfg", "type": "JOB"},
                "uuid": "aaa-bb-ccc",
                "version": "0",
            },
        ),
        RuleTest(
            name="Threat Detected Event - Deny",
            expected_result=True,
            log={
                "actor": {"alternateId": "unknown", "displayName": "1.2.3.4", "id": "1.2.3.4", "type": "IP address"},
                "authenticationcontext": {"authenticationStep": 0},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Dallas",
                        "country": "United States",
                        "geolocation": {"lat": 32.7908, "lon": -96.8336},
                        "postalCode": "75207",
                        "state": "Texas",
                    },
                    "ipAddress": "1.2.3.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Windows 10",
                        "rawUserAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "dtHash": "abcsadfjsald",
                        "requestId": "alsdjflasf",
                        "requestUri": "/oauth2/v1/authorize",
                        "threatDetections": '{"Login failures with high unknown users count":"HIGH","Password Spray":"HIGH","Login Failures":"MEDIUM"}',
                        "threatSuspected": "true",
                        "url": "/oauth2/v1/authorize",
                    },
                },
                "displaymessage": "Request from suspicious actor",
                "eventtype": "security.threat.detected",
                "legacyeventtype": "security.threat.detected",
                "outcome": {
                    "reason": "Password Spray, Login failures with high unknown users count, Login Failures",
                    "result": "DENY",
                },
                "published": "2022-12-14 19:16:32.015",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Dallas",
                                "country": "United States",
                                "geolocation": {"lat": 32.7908, "lon": -96.8336},
                                "postalCode": "75207",
                                "state": "Texas",
                                "ip": "1.2.3.4",
                                "version": "V4",
                            },
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 62240,
                    "asOrg": "packethub s.a.",
                    "domain": ".",
                    "isProxy": False,
                    "isp": "clouvider limited",
                },
                "severity": "WARN",
                "transaction": {"detail": {}, "id": "asdfjaslf", "type": "WEB"},
                "uuid": "asdfa-1234-asdfdas",
                "version": "0",
            },
        ),
        RuleTest(
            name="Threat Detected Event - Success",
            expected_result=True,
            log={
                "actor": {"alternateId": "unknown", "displayName": "1.2.3.4", "id": "1.2.3.4", "type": "IP address"},
                "authenticationcontext": {"authenticationStep": 0},
                "client": {
                    "device": "Computer",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Windows 10",
                        "rawUserAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "dtHash": "abcsadfjsald",
                        "requestId": "alsdjflasf",
                        "requestUri": "/oauth2/v1/authorize",
                        "threatDetections": '{"Login failures with high unknown users count":"HIGH","Password Spray":"HIGH","Login Failures":"MEDIUM"}',
                        "threatSuspected": "true",
                        "url": "/oauth2/v1/authorize",
                    },
                },
                "displaymessage": "Request from suspicious actor",
                "eventtype": "security.threat.detected",
                "legacyeventtype": "security.threat.detected",
                "outcome": {
                    "reason": "Password Spray, Login failures with high unknown users count, Login Failures",
                    "result": "SUCCESS",
                },
                "published": "2022-12-14 19:16:32.015",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Dallas",
                                "country": "United States",
                                "geolocation": {"lat": 32.7908, "lon": -96.8336},
                                "postalCode": "75207",
                                "state": "Texas",
                                "ip": "1.2.3.4",
                                "version": "V4",
                            },
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 62240,
                    "asOrg": "packethub s.a.",
                    "domain": ".",
                    "isProxy": False,
                    "isp": "clouvider limited",
                },
                "severity": "WARN",
                "transaction": {"detail": {}, "id": "asdfjaslf", "type": "WEB"},
                "uuid": "asdfa-1234-asdfdas",
                "version": "0",
            },
        ),
    ]



from pypanther.rules.panther_audit.panther_detection_deleted import PantherDetectionDeleted as PantherDetectionDeleted
from pypanther.rules.panther_audit.panther_saml_modified import PantherSAMLModified as PantherSAMLModified
from pypanther.rules.panther_audit.panther_sensitive_role_created import PantherSensitiveRole as PantherSensitiveRole
from pypanther.rules.panther_audit.panther_user_modified import PantherUserModified as PantherUserModified



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.slack import slack_alert_context


@panther_managed
class SlackAuditLogsInformationBarrierModified(Rule):
    id = "Slack.AuditLogs.InformationBarrierModified-prototype"
    display_name = "Slack Information Barrier Modified"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Defense Evasion", "Impair Defenses", "Disable or Modify Tools"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562.001"]}
    default_severity = Severity.MEDIUM
    default_description = "Detects when a Slack information barrier is deleted/updated"
    default_reference = "https://slack.com/intl/en-gb/help/articles/360056171734-Create-information-barriers-in-Slack"
    summary_attributes = ["action", "p_any_ip_addresses", "p_any_emails"]
    INFORMATION_BARRIER_ACTIONS = {
        "barrier_deleted": "Slack Information Barrier Deleted",
        "barrier_updated": "Slack Information Barrier Updated",
    }

    def rule(self, event):
        return event.get("action") in self.INFORMATION_BARRIER_ACTIONS

    def title(self, event):
        if event.get("action") in self.INFORMATION_BARRIER_ACTIONS:
            return self.INFORMATION_BARRIER_ACTIONS.get(event.get("action"))
        return "Slack Information Barrier Modified"

    def alert_context(self, event):
        return slack_alert_context(event)

    tests = [
        RuleTest(
            name="Information Barrier Deleted",
            expected_result=True,
            log={
                "action": "barrier_deleted",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "A012B3CDEFG",
                        "name": "username",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace",
                        "id": "T01234N56GB",
                        "name": "test-workspace",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
            },
        ),
        RuleTest(
            name="Information Barrier Updated",
            expected_result=True,
            log={
                "action": "barrier_updated",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "A012B3CDEFG",
                        "name": "username",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace",
                        "id": "T01234N56GB",
                        "name": "test-workspace",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
            },
        ),
        RuleTest(
            name="User Logout",
            expected_result=False,
            log={
                "action": "user_logout",
                "actor": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "context": {
                    "ip_address": "1.2.3.4",
                    "location": {
                        "domain": "test-workspace-1",
                        "id": "T01234N56GB",
                        "name": "test-workspace-1",
                        "type": "workspace",
                    },
                    "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
                },
                "date_create": "2022-07-28 15:22:32",
                "entity": {
                    "type": "user",
                    "user": {
                        "email": "user@example.com",
                        "id": "W012J3FEWAU",
                        "name": "primary-owner",
                        "team": "T01234N56GB",
                    },
                },
                "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
            },
        ),
    ]



"""
Used to define global helpers for Wiz events
"""


def wiz_success(event):
    if event.get("status", "") == "SUCCESS":
        return True
    return False


def wiz_alert_context(event):
    return {
        "action": event.get("action", ""),
        "actor": wiz_actor(event),
        "source_ip": event.get("sourceip", ""),
        "event_id": event.get("id", ""),
        "action_parameters": event.get("actionparameters", ""),
    }


def wiz_actor(event):
    user = event.get("user")
    serviceaccount = event.get("serviceAccount")

    if user is not None:
        return {
            "type": "user",
            "id": user.get("id"),
            "name": user.get("name"),
        }

    if serviceaccount is not None:
        return {
            "type": "serviceaccount",
            "id": serviceaccount.get("id"),
            "name": serviceaccount.get("name"),
        }

    return {
        "type": "unknown",
        "id": "<Unknown ID>",
        "name": "<Unknown Name>",
    }



from pypanther.rules.azure_signin.azure_failed_signins import AzureAuditManyFailedSignIns as AzureAuditManyFailedSignIns
from pypanther.rules.azure_signin.azure_invite_external_users import (
    AzureAuditInviteExternalUsers as AzureAuditInviteExternalUsers,
)
from pypanther.rules.azure_signin.azure_legacyauth import AzureAuditLegacyAuth as AzureAuditLegacyAuth
from pypanther.rules.azure_signin.azure_mfa_disabled import AzureAuditMFADisabled as AzureAuditMFADisabled
from pypanther.rules.azure_signin.azure_policy_changed import AzureAuditPolicyChanged as AzureAuditPolicyChanged
from pypanther.rules.azure_signin.azure_risklevel_passthrough import (
    AzureAuditRiskLevelPassthrough as AzureAuditRiskLevelPassthrough,
)
from pypanther.rules.azure_signin.azure_role_changed_pim import AzureAuditRoleChangedPIM as AzureAuditRoleChangedPIM



from pypanther.rules.mongodb.mongodb_2fa_disabled import MongoDB2FADisabled as MongoDB2FADisabled
from pypanther.rules.mongodb.mongodb_access_allowed_from_anywhere import (
    MongoDBAccessAllowedFromAnywhere as MongoDBAccessAllowedFromAnywhere,
)
from pypanther.rules.mongodb.mongodb_alerting_disabled import (
    MongoDBAlertingDisabledOrDeleted as MongoDBAlertingDisabledOrDeleted,
)
from pypanther.rules.mongodb.mongodb_atlas_api_key_created import MongoDBAtlasApiKeyCreated as MongoDBAtlasApiKeyCreated
from pypanther.rules.mongodb.mongodb_external_user_invited import (
    MongoDBExternalUserInvited as MongoDBExternalUserInvited,
)
from pypanther.rules.mongodb.mongodb_external_user_invited_no_config import (
    MongoDBExternalUserInvitedNoConfig as MongoDBExternalUserInvitedNoConfig,
)
from pypanther.rules.mongodb.mongodb_identity_provider_activity import (
    MongoDBIdentityProviderActivity as MongoDBIdentityProviderActivity,
)
from pypanther.rules.mongodb.mongodb_logging_toggled import MongoDBLoggingToggled as MongoDBLoggingToggled
from pypanther.rules.mongodb.mongodb_org_membership_restriction_disabled import (
    MongoDBorgMembershipRestrictionDisabled as MongoDBorgMembershipRestrictionDisabled,
)
from pypanther.rules.mongodb.mongodb_user_created_or_deleted import (
    MongoDBUserCreatedOrDeleted as MongoDBUserCreatedOrDeleted,
)
from pypanther.rules.mongodb.mongodb_user_roles_changed import MongoDBUserRolesChanged as MongoDBUserRolesChanged



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.crowdstrike_fdr import crowdstrike_network_detection_alert_context


@panther_managed
class ConnectiontoEmbargoedCountry(Rule):
    default_description = "Detection to alert when internal asset is communicating with an sanctioned destination. This detection leverages Panther UDM and IPInfo enrichment."
    default_reference = "U.S. Sanctioned Destinations - https://www.bis.doc.gov/index.php/policy-guidance/country-guidance/sanctioned-destinations"
    display_name = "Connection to Embargoed Country"
    log_types = [LogType.CROWDSTRIKE_FDR_EVENT]
    id = "Connection.to.Embargoed.Country-prototype"
    default_severity = Severity.LOW
    # U.S. Gov Sanctioned Destinations
    # Cuba
    # Iran
    # DPRK
    # Syria
    EMBARGO_COUNTRY_CODES = {"CU", "IR", "KP", "SY"}

    def get_enrichment_obj(self, event):
        return event.deep_get("p_enrichment", "ipinfo_location", "p_any_ip_addresses", default=None)

    def rule(self, event):
        enrichment_obj = self.get_enrichment_obj(event)
        # enrichment_object returns a list.
        # Iterate over list and check if the "country" value matches the country codes.
        if enrichment_obj:
            for i in enrichment_obj:
                if i.get("country") in self.EMBARGO_COUNTRY_CODES:
                    return True
        return False

    def title(self, event):
        enrichment_obj = self.get_enrichment_obj(event)
        country_codes = set(i.get("country") for i in enrichment_obj if i.get("country") in self.EMBARGO_COUNTRY_CODES)
        return f"Connection made to embargoed country: [{country_codes}]."

    def alert_context(self, event):
        if event.get("p_log_type") == "Crowdstrike.FDREvent":
            return crowdstrike_network_detection_alert_context(event) | {
                "p_any_ip_addresses": event.get("p_any_ip_addresses"),
            }
        return {"p_any_ip_addresses": event.get("p_any_ip_addresses")}

    tests = [
        RuleTest(
            name="Connection To CU",
            expected_result=True,
            log={
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "1431649125",
                "ContextProcessId": "1685738",
                "ContextTimeStamp": "2023-04-28 18:49:37.731",
                "Entitlements": "15",
                "InContext": "0",
                "aid": "877761efa8db44d7redacted",
                "aip": "1.1.1.1",
                "cid": "cfe6986909644340redacted",
                "event": {
                    "ConfigBuild": "1007.3.0016606.11",
                    "ConfigStateHash": "1431649125",
                    "ConnectionDirection": "0",
                    "ConnectionFlags": "0",
                    "ContextProcessId": "1685738",
                    "ContextTimeStamp": "1682707777.731",
                    "EffectiveTransmissionClass": "3",
                    "Entitlements": "15",
                    "EventOrigin": "1",
                    "InContext": "0",
                    "LocalAddressIP4": "10.0.0.1",
                    "LocalPort": "137",
                    "Protocol": "17",
                    "RemoteAddressIP4": "152.206.0.1",
                    "RemotePort": "443",
                    "aid": "877761efa8db44d7redacted",
                    "aip": "1.1.1.1",
                    "cid": "cfe6986909644340redacted",
                    "event_platform": "Win",
                    "event_simpleName": "NetworkConnectIP4",
                    "id": "34019b0c-c7de-4725-9f93-4b8d16688673",
                    "name": "NetworkConnectIP4V12",
                    "timestamp": "1682707778681",
                },
                "event_platform": "Win",
                "event_simpleName": "NetworkConnectIP4",
                "fdr_event_type": "NetworkConnectIP4",
                "id": "34019b0c-c7de-4725-9f93-4b8d16688673",
                "name": "NetworkConnectIP4V12",
                "p_any_ip_addresses": ["152.206.0.1", "10.0.0.1", "1.1.1.1"],
                "p_enrichment": {
                    "greynoise_riot_advanced": {
                        "p_any_ip_addresses": [
                            {
                                "ip_cidr": "1.1.1.1/32",
                                "provider": {
                                    "category": "public_dns",
                                    "description": "Cloudflare, Inc. is an American...",
                                    "explanation": "Public DNS services are used as...",
                                    "name": "Cloudflare Public DNS",
                                    "precedence": 0,
                                    "trust_level": "1",
                                },
                                "scan_time": "2023-04-28 21:11:03.820349735",
                            },
                        ],
                    },
                    "ipinfo_asn": {
                        "p_any_ip_addresses": [
                            {
                                "asn": "AS27725",
                                "domain": "etecsa.cu",
                                "name": "Empresa de Telecomunicaciones de Cuba, S.A.",
                                "route": "152.206.0.0/17",
                                "type": "isp",
                            },
                            {
                                "asn": "AS13335",
                                "domain": "cloudflare.com",
                                "name": "Cloudflare, Inc.",
                                "route": "1.1.1.0/24",
                                "type": "hosting",
                            },
                        ],
                    },
                    "ipinfo_location": {
                        "p_any_ip_addresses": [
                            {
                                "city": "Matanzas",
                                "country": "CU",
                                "lat": "23.04111",
                                "lng": "-81.5775",
                                "postal_code": "",
                                "region": "Matanzas Province",
                                "region_code": "04",
                                "timezone": "America/Havana",
                            },
                            {
                                "city": "Los Angeles",
                                "country": "US",
                                "lat": "34.0522",
                                "lng": "-118.2437",
                                "postal_code": "90076",
                                "region": "California",
                                "region_code": "CA",
                                "timezone": "America/Los_Angeles",
                            },
                        ],
                    },
                    "ipinfo_privacy": {
                        "p_any_ip_addresses": [
                            {
                                "hosting": True,
                                "proxy": False,
                                "relay": False,
                                "service": "",
                                "tor": False,
                                "vpn": False,
                            },
                        ],
                    },
                },
                "p_log_type": "Crowdstrike.FDREvent",
                "timestamp": "2023-04-28 18:49:38.681",
            },
        ),
        RuleTest(
            name="Google DNS",
            expected_result=False,
            log={
                "p_any_ip_addresses": ["8.8.8.8"],
                "p_enrichment": {
                    "ipinfo_location": {
                        "p_any_ip_addresses": [
                            {
                                "city": "Mountain View",
                                "country": "US",
                                "lat": "37.4056",
                                "lng": "-122.0775",
                                "postal_code": "94043",
                                "region": "California",
                                "region_code": "CA",
                                "timezone": "America/Los_Angeles",
                            },
                        ],
                    },
                },
                "p_log_type": "Crowdstrike.FDREvent",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.zendesk import ZENDESK_CHANGE_DESCRIPTION


@panther_managed
class ZendeskMobileAppAccessUpdated(Rule):
    id = "Zendesk.MobileAppAccessUpdated-prototype"
    display_name = "Zendesk Mobile App Access Modified"
    log_types = [LogType.ZENDESK_AUDIT]
    tags = ["Zendesk", "Persistence:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0003:T1078"]}
    default_severity = Severity.MEDIUM
    default_description = "A user updated account setting that enabled or disabled mobile app access."
    default_reference = "https://support.zendesk.com/hc/en-us/articles/4408846407066-About-the-Zendesk-Support-mobile-app#:~:text=More%20settings.-,Configuring%20the%20mobile%20app,-Activate%20the%20new"
    summary_attributes = ["p_any_ip_addresses"]
    MOBILE_APP_ACTIONS = {"create", "update"}

    def rule(self, event):
        return (
            event.get("source_type") == "account_setting"
            and event.get("action", "") in self.MOBILE_APP_ACTIONS
            and (event.get("source_label", "") == "Zendesk Support Mobile App Access")
        )

    def title(self, event):
        action = event.get(ZENDESK_CHANGE_DESCRIPTION, "<UNKNOWN_ACTION>")
        return f"User [{event.udm('actor_user')}] {action} mobile app access"

    def severity(self, event):
        if event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower() == "disabled":
            return "INFO"
        return "MEDIUM"

    tests = [
        RuleTest(
            name="Zendesk - Mobile App Access Off",
            expected_result=True,
            log={
                "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
                "id": 123456789123,
                "action_label": "Updated",
                "actor_id": 123,
                "actor_name": "John Doe",
                "source_id": 123,
                "source_type": "account_setting",
                "source_label": "Zendesk Support Mobile App Access",
                "action": "create",
                "change_description": "Disabled",
                "ip_address": "127.0.0.1",
                "created_at": "2021-05-28T18:39:50Z",
                "p_log_type": "Zendesk.Audit",
            },
        ),
        RuleTest(
            name="Zendesk - Mobile App Access On",
            expected_result=True,
            log={
                "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
                "id": 123456789123,
                "action_label": "Updated",
                "actor_id": 123,
                "actor_name": "John Doe",
                "source_id": 123,
                "source_type": "account_setting",
                "source_label": "Zendesk Support Mobile App Access",
                "action": "create",
                "change_description": "Enabled",
                "ip_address": "127.0.0.1",
                "created_at": "2021-05-28T18:39:50Z",
                "p_log_type": "Zendesk.Audit",
            },
        ),
        RuleTest(
            name="Zendesk - Credit Card Redaction",
            expected_result=False,
            log={
                "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
                "id": 123456789123,
                "action_label": "Updated",
                "actor_id": 123,
                "actor_name": "John Doe",
                "source_id": 123,
                "source_type": "account_setting",
                "source_label": "Credit Card Redaction",
                "action": "create",
                "change_description": "Enabled",
                "ip_address": "127.0.0.1",
                "created_at": "2021-05-28T18:39:50Z",
                "p_log_type": "Zendesk.Audit",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.okta import okta_alert_context


@panther_managed
class OktaPasswordExtractionviaSCIM(Rule):
    id = "Okta.Password.Extraction.via.SCIM-prototype"
    display_name = "Okta Cleartext Passwords Extracted via SCIM Application"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    reports = {"MITRE ATT&CK": ["TA0006:T1556"]}
    default_severity = Severity.HIGH
    default_description = "An application admin has extracted cleartext user passwords via SCIM app. Malcious actors can extract plaintext passwords by creating a SCIM application under their control and configuring it to sync passwords from Okta.\n"
    default_reference = (
        "https://www.authomize.com/blog/authomize-discovers-password-stealing-and-impersonation-risks-to-in-okta/\n"
    )
    dedup_period_minutes = 30

    def rule(self, event):
        return event.get("eventType") == "application.lifecycle.update" and "Pushing user passwords" in event.deep_get(
            "outcome",
            "reason",
            default="",
        )

    def title(self, event):
        target = event.deep_walk("target", "alternateId", default="<alternateId-not-found>", return_val="first")
        return f"{event.deep_get('actor', 'displayName', default='<displayName-not-found>')} <{event.deep_get('actor', 'alternateId', default='alternateId-not-found')}> extracted cleartext user passwords via SCIM app [{target}]"

    def alert_context(self, event):
        return okta_alert_context(event)

    tests = [
        RuleTest(
            name="Other Event",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 20, "lon": -25},
                        "postalCode": "12345",
                        "state": "Ohio",
                    },
                    "ipAddress": "1.3.2.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "requestId": "AbCdEf12G",
                        "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                        "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                    },
                },
                "displaymessage": "Authentication of user via MFA",
                "eventtype": "user.authentication.auth_via_mfa",
                "legacyeventtype": "core.user.factor.attempt_fail",
                "outcome": {"result": "SUCCESS"},
                "published": "2022-06-22 18:18:29.015",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Springfield",
                                "country": "United States",
                                "geolocation": {"lat": 20, "lon": -25},
                                "postalCode": "12345",
                                "state": "Ohio",
                            },
                            "ip": "1.3.2.4",
                            "version": "V4",
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 701,
                    "asOrg": "verizon",
                    "domain": "verizon.net",
                    "isProxy": False,
                    "isp": "verizon",
                },
                "severity": "INFO",
                "target": [
                    {
                        "alternateId": "peter.griffin@company.com",
                        "displayName": "Peter Griffin",
                        "id": "0002222AAAA",
                        "type": "User",
                    },
                ],
                "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
                "uuid": "AbC-123-XyZ",
                "version": "0",
            },
        ),
        RuleTest(
            name="FastPass Phishing Block Event",
            expected_result=True,
            log={
                "actor": {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "00abc123",
                    "type": "User",
                },
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Springfield",
                        "country": "United States",
                        "geolocation": {"lat": 20, "lon": -25},
                        "postalCode": "12345",
                        "state": "Ohio",
                    },
                    "ipAddress": "1.3.2.4",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugcontext": {
                    "debugData": {
                        "requestId": "AbCdEf12G",
                        "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                        "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                    },
                },
                "displaymessage": "Authentication of user via MFA",
                "eventtype": "application.lifecycle.update",
                "legacyeventtype": "core.user.factor.attempt_fail",
                "outcome": {"reason": "Pushing user passwords", "result": "SUCCESS"},
                "published": "2022-06-22 18:18:29.015",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Springfield",
                                "country": "United States",
                                "geolocation": {"lat": 20, "lon": -25},
                                "postalCode": "12345",
                                "state": "Ohio",
                            },
                            "ip": "1.3.2.4",
                            "version": "V4",
                        },
                    ],
                },
                "securitycontext": {
                    "asNumber": 701,
                    "asOrg": "verizon",
                    "domain": "verizon.net",
                    "isProxy": False,
                    "isp": "verizon",
                },
                "severity": "INFO",
                "target": [
                    {
                        "alternateId": "peter.griffin@company.com",
                        "displayName": "Peter Griffin",
                        "id": "0002222AAAA",
                        "type": "User",
                    },
                ],
                "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
                "uuid": "AbC-123-XyZ",
                "version": "0",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import pattern_match


@panther_managed
class AWSMacieEvasion(Rule):
    id = "AWS.Macie.Evasion-prototype"
    display_name = "AWS Macie Disabled/Updated"
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.MEDIUM
    default_description = "Amazon Macie is a data security and data privacy service to discover and protect sensitive data. Security teams use Macie to detect open S3 Buckets that could have potentially sensitive data in it along with policy violations, such as missing Encryption. If an attacker disables Macie, it could potentially hide data exfiltration.\n"
    default_reference = "https://aws.amazon.com/macie/"
    default_runbook = "Analyze the events to ensure it's not normal maintenance. If it's abnormal, run the Indicator Search on the UserIdentity:Arn for the past hour and analyze other services accessed/changed.\n"
    threshold = 5
    summary_attributes = [
        "awsRegion",
        "eventName",
        "p_any_aws_arns",
        "p_any_ip_addresses",
        "userIdentity:type",
        "userIdentity:arn",
    ]
    MACIE_EVENTS = {
        "ArchiveFindings",
        "CreateFindingsFilter",
        "DeleteMember",
        "DisassociateFromMasterAccount",
        "DisassociateMember",
        "DisableMacie",
        "DisableOrganizationAdminAccount",
        "UpdateFindingsFilter",
        "UpdateMacieSession",
        "UpdateMemberSession",
        "UpdateClassificationJob",
    }

    def rule(self, event):
        return event.get("eventName") in self.MACIE_EVENTS and pattern_match(
            event.get("eventSource"),
            "macie*.amazonaws.com",
        )

    def title(self, event):
        account = event.get("recipientAccountId")
        user_arn = event.deep_get("userIdentity", "arn")
        return f"AWS Macie in AWS Account [{account}] Disabled/Updated by [{user_arn}]"

    tests = [
        RuleTest(
            name="ListMembers",
            expected_result=False,
            log={
                "awsRegion": "us-west-1",
                "eventCategory": "Management",
                "eventID": "5b3e4cf6-c37d-4c8c-9016-b8444a37ceaa",
                "eventName": "ListMembers",
                "eventSource": "macie2.amazonaws.com",
                "eventTime": "2022-09-27 18:11:33",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_any_aws_account_ids": ["123456789012"],
                "p_any_aws_arns": [
                    "arn:aws:iam::123456789012:role/Admin",
                    "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                ],
                "p_any_ip_addresses": ["178.253.78.209"],
                "p_any_trace_ids": ["AAAASSSST64ZTHFY7777"],
                "p_event_time": "2022-09-27 18:11:33",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-09-27 18:16:43.428",
                "p_row_id": "665d45a409cad7d68ff7bbd4138d02",
                "p_source_id": "b00eb354-da7a-49dd-9cc6-32535e32096a",
                "p_source_label": "CloudTrail Test",
                "readOnly": True,
                "recipientAccountId": "123456789012",
                "requestID": "2164bbea-3eb0-444b-8e10-8ba53b3460b6",
                "requestParameters": {"maxResults": "1", "onlyAssociated": "true"},
                "sourceIPAddress": "178.253.78.209",
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36",
                "userIdentity": {
                    "accessKeyId": "AAAASSSST64ZTHFY7777",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                    "principalId": "AAAAA44444LE6DYFKKKKK:Jack",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-09-27T17:56:01Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/Admin",
                            "principalId": "AAAAA44444LE6DYFKKKKK",
                            "type": "Role",
                            "userName": "Admin",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="UpdateSession",
            expected_result=True,
            log={
                "awsRegion": "us-east-2",
                "eventCategory": "Management",
                "eventID": "63033dfd-08c9-42f3-80ae-dca45e86ae84",
                "eventName": "UpdateMacieSession",
                "eventSource": "macie2.amazonaws.com",
                "eventTime": "2022-09-27 19:59:08",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_any_aws_account_ids": ["123456789012"],
                "p_any_aws_arns": [
                    "arn:aws:iam::123456789012:role/Admin",
                    "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                ],
                "p_any_ip_addresses": ["46.91.25.204"],
                "p_any_trace_ids": ["ASIASWJRT64Z42HFV6QX"],
                "p_event_time": "2022-09-27 19:59:08",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-09-27 20:02:43.816",
                "p_row_id": "665d45a409cad7d68ff7bbd4138123",
                "p_source_id": "b00eb354-da7a-49dd-9cc6-32535e32096a",
                "p_source_label": "CloudTrail Test",
                "readOnly": False,
                "recipientAccountId": "123456789012",
                "requestID": "1b9981dc-21d2-4f77-92b0-69e23c8a40de",
                "requestParameters": {"findingPublishingFrequency": "SIX_HOURS"},
                "sourceIPAddress": "46.91.25.204",
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36",
                "userIdentity": {
                    "accessKeyId": "ASIASWJRT64Z42HFV6QX",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                    "principalId": "AAAAA44444LE6DYFKKKKK:Jack",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-09-27T17:56:01Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/Admin",
                            "principalId": "AAAAA44444LE6DYFKKKKK",
                            "type": "Role",
                            "userName": "Admin",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="UpdateSession (Macie v1 event)",
            expected_result=True,
            log={
                "awsRegion": "us-east-2",
                "eventCategory": "Management",
                "eventID": "63033dfd-08c9-42f3-80ae-dca45e86ae84",
                "eventName": "UpdateMacieSession",
                "eventSource": "macie.amazonaws.com",
                "eventTime": "2022-09-27 19:59:08",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_any_aws_account_ids": ["123456789012"],
                "p_any_aws_arns": [
                    "arn:aws:iam::123456789012:role/Admin",
                    "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                ],
                "p_any_ip_addresses": ["46.91.25.204"],
                "p_any_trace_ids": ["ASIASWJRT64Z42HFV6QX"],
                "p_event_time": "2022-09-27 19:59:08",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2022-09-27 20:02:43.816",
                "p_row_id": "665d45a409cad7d68ff7bbd4138123",
                "p_source_id": "b00eb354-da7a-49dd-9cc6-32535e32096a",
                "p_source_label": "CloudTrail Test",
                "readOnly": False,
                "recipientAccountId": "123456789012",
                "requestID": "1b9981dc-21d2-4f77-92b0-69e23c8a40de",
                "requestParameters": {"findingPublishingFrequency": "SIX_HOURS"},
                "sourceIPAddress": "46.91.25.204",
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36",
                "userIdentity": {
                    "accessKeyId": "ASIASWJRT64Z42HFV6QX",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                    "principalId": "AAAAA44444LE6DYFKKKKK:Jack",
                    "sessionContext": {
                        "attributes": {"creationDate": "2022-09-27T17:56:01Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/Admin",
                            "principalId": "AAAAA44444LE6DYFKKKKK",
                            "type": "Role",
                            "userName": "Admin",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.msft import msft_graph_alert_context


@panther_managed
class MicrosoftGraphPassthrough(Rule):
    default_description = "The Microsoft Graph security API federates queries to all onboarded security providers, including Azure AD Identity Protection, Microsoft 365, Microsoft Defender (Cloud, Endpoint, Identity) and Microsoft Sentinel"
    default_reference = "https://learn.microsoft.com/en-us/graph/api/resources/security-api-overview"
    display_name = "Microsoft Graph Passthrough"
    default_severity = Severity.MEDIUM
    log_types = [LogType.MICROSOFT_GRAPH_SECURITY_ALERT]
    id = "Microsoft.Graph.Passthrough-prototype"

    def rule(self, event):
        return event.get("status") == "newAlert" and event.get("severity", "").lower() != "informational"

    def title(self, event):
        return f"Microsoft Graph Alert ({event.get('title')})"

    def dedup(self, event):
        return event.get("id")

    def severity(self, event):
        if event.get("severity", "").lower() == "informational":
            return "INFO"
        return event.get("severity")

    def alert_context(self, event):
        return msft_graph_alert_context(event)

    tests = [
        RuleTest(
            name="Anonymous Login Event",
            expected_result=True,
            log={
                "azuretenantid": "12345-abcde-a1b2k3",
                "category": "AnonymousLogin",
                "createddatetime": "2022-08-04 14:31:48.438",
                "description": "Sign-in from an anonymous IP address (e.g. Tor browser, anonymizer VPNs)",
                "eventdatetime": "2022-08-04 14:31:48.438",
                "id": "abcd12345efghijk6789",
                "lastmodifieddatetime": "2022-08-04 14:34:56.229",
                "severity": "medium",
                "status": "newAlert",
                "title": "Anonymous IP address",
                "userstates": [
                    {
                        "aadUserId": "011d5ede-0faa-4946-a25e-b2cd0c47a52c",
                        "accountName": "homer.simpson",
                        "domainName": "corporation.onmicrosoft.com",
                        "emailRole": "unknown",
                        "logonDateTime": "2022-08-04 14:31:48.438699700",
                        "logonIp": "185.220.103.6",
                        "logonLocation": "Brooklyn, New York, US",
                        "userPrincipalName": "homer.simpson@corporation.onmicrosoft.com",
                    },
                ],
                "vendorinformation": {"provider": "IPC", "vendor": "Microsoft"},
            },
        ),
        RuleTest(
            name="Password Spray Event",
            expected_result=True,
            log={
                "azuretenantid": "abcdef-123456-ghijklmn",
                "category": "PasswordSpray",
                "createddatetime": "2022-08-17 09:28:04.767",
                "description": "Password spray attack detected",
                "eventdatetime": "2022-08-16 14:22:00.698",
                "id": "abcdefg-123456-hijklmno",
                "lastmodifieddatetime": "2022-08-17 14:30:21.979",
                "severity": "high",
                "status": "newAlert",
                "title": "Password Spray",
                "userstates": [
                    {
                        "aadUserId": "abcdefg-123456-lmnop",
                        "accountName": "homer.simpson",
                        "domainName": "corporation.onmicrosoft.com",
                        "emailRole": "unknown",
                        "logonDateTime": "2022-08-16 14:22:00.698619500",
                        "logonIp": "109.70.100.21",
                        "logonLocation": "San Francisco, CA",
                        "userPrincipalName": "homer.simpson@corporation.onmicrosoft.com",
                    },
                ],
                "vendorinformation": {"provider": "IPC", "vendor": "Microsoft"},
            },
        ),
        RuleTest(
            name="Resolved Event",
            expected_result=False,
            log={
                "azuretenantid": "abcdefg-12345",
                "category": "AnonymousLogin",
                "createddatetime": "2022-09-12 19:54:13.725",
                "description": "Sign-in from an anonymous IP address (e.g. Tor browser, anonymizer VPNs)",
                "eventdatetime": "2022-09-12 19:54:13.725",
                "id": "abcdefg12345hijklmnop",
                "lastmodifieddatetime": "2022-09-12 19:56:57.833",
                "severity": "high",
                "status": "resolved",
                "title": "Anonymous IP address",
                "userstates": [
                    {
                        "aadUserId": "abcdefg1234",
                        "accountName": "homer.simpson",
                        "domainName": "corporation.onmicrosoft.com",
                        "emailRole": "unknown",
                        "logonDateTime": "2022-09-12 19:54:13.725589800",
                        "logonIp": "109.70.100.35",
                        "logonLocation": "San Francisco, CA",
                        "userPrincipalName": "homer.simpson@corporation.onmicrosoft.com",
                    },
                ],
                "vendorinformation": {"provider": "IPC", "vendor": "Microsoft"},
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class PushSecurityPhishingAttack(Rule):
    id = "Push.Security.Phishing.Attack-prototype"
    display_name = "Push Security Phishing Attack"
    log_types = [LogType.PUSH_SECURITY_CONTROLS]
    default_severity = Severity.HIGH

    def rule(self, event):
        if event.get("object") == "PASSWORD_PHISHING":
            return True
        return False

    def severity(self, event):
        if event.deep_get("new", "mode") != "BLOCK":
            return "HIGH"
        return "LOW"

    def title(self, event):
        app_type = event.deep_get("new", "appType")
        employee_email = event.deep_get("new", "employee", "email")
        new_mode = event.deep_get("new", "mode")
        return (
            f"Phishing attack on app {app_type} user {employee_email}.              Attack detected in mode {new_mode}."
        )

    tests = [
        RuleTest(
            name="Phishing Detected - Block Mode",
            expected_result=True,
            log={
                "id": "c478966c-f927-411c-b919-179832d3d50c",
                "new": {
                    "appType": "OKTA",
                    "browser": "CHROME",
                    "employee": {
                        "chatopsEnabled": True,
                        "creationTimestamp": 1698669223.0,
                        "department": "Security Engineering",
                        "email": "john.hill@example.com",
                        "firstName": "John",
                        "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                        "lastName": "Hill",
                        "licensed": True,
                        "location": "New York",
                    },
                    "mode": "BLOCK",
                    "os": "WINDOWS",
                    "referrerUrl": "https://statics.teams.cdn.office.net/",
                    "sourceIpAddress": "8.158.25.38",
                    "url": "https://evil.com/okta.php",
                    "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 Edge/16.16299",
                },
                "object": "PASSWORD_PHISHING",
                "timestamp": 1698604061.0,
                "version": "1",
            },
        ),
        RuleTest(
            name="Phishing Detected - Monitor Mode",
            expected_result=True,
            log={
                "id": "c478966c-f927-411c-b919-179832d3d50c",
                "new": {
                    "appType": "OKTA",
                    "browser": "CHROME",
                    "employee": {
                        "chatopsEnabled": True,
                        "creationTimestamp": 1698669223.0,
                        "department": "Security Engineering",
                        "email": "john.hill@example.com",
                        "firstName": "John",
                        "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                        "lastName": "Hill",
                        "licensed": True,
                        "location": "New York",
                    },
                    "mode": "MONITOR",
                    "os": "WINDOWS",
                    "referrerUrl": "https://statics.teams.cdn.office.net/",
                    "sourceIpAddress": "8.158.25.38",
                    "url": "https://evil.com/okta.php",
                    "userAgent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 Edge/16.16299",
                },
                "object": "PASSWORD_PHISHING",
                "timestamp": 1698604061.0,
                "version": "1",
            },
        ),
    ]



from pypanther.rules.snowflake.snowflake_stream_account_admin_assigned import (
    SnowflakeStreamAccountAdminGranted as SnowflakeStreamAccountAdminGranted,
)
from pypanther.rules.snowflake.snowflake_stream_brute_force_by_ip import (
    SnowflakeStreamBruteForceByIp as SnowflakeStreamBruteForceByIp,
)
from pypanther.rules.snowflake.snowflake_stream_brute_force_by_username import (
    SnowflakeStreamBruteForceByUsername as SnowflakeStreamBruteForceByUsername,
)
from pypanther.rules.snowflake.snowflake_stream_external_shares import (
    SnowflakeStreamExternalShares as SnowflakeStreamExternalShares,
)
from pypanther.rules.snowflake.snowflake_stream_file_downloaded import (
    SnowflakeStreamFileDownloaded as SnowflakeStreamFileDownloaded,
)
from pypanther.rules.snowflake.snowflake_stream_login_success import (
    SnowflakeStreamLoginSuccess as SnowflakeStreamLoginSuccess,
)
from pypanther.rules.snowflake.snowflake_stream_login_without_mfa import (
    SnowflakeStreamLoginWithoutMFA as SnowflakeStreamLoginWithoutMFA,
)
from pypanther.rules.snowflake.snowflake_stream_public_role_grant import (
    SnowflakeStreamPublicRoleGrant as SnowflakeStreamPublicRoleGrant,
)
from pypanther.rules.snowflake.snowflake_stream_table_copied_into_stage import (
    SnowflakeStreamTableCopiedIntoStage as SnowflakeStreamTableCopiedIntoStage,
)
from pypanther.rules.snowflake.snowflake_stream_temp_stage_created import (
    SnowflakeStreamTempStageCreated as SnowflakeStreamTempStageCreated,
)
from pypanther.rules.snowflake.snowflake_stream_user_created import (
    SnowflakeStreamUserCreated as SnowflakeStreamUserCreated,
)
from pypanther.rules.snowflake.snowflake_stream_user_enabled import (
    SnowflakeStreamUserEnabled as SnowflakeStreamUserEnabled,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class SigninwithAWSCLIprompt(Rule):
    id = "Sign-in.with.AWS.CLI.prompt-prototype"
    display_name = "SIGNAL - Sign-in with AWS CLI prompt"
    create_alert = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO

    def rule(self, event):
        return event.get("eventSource") == "sso.amazonaws.com" and event.get("eventName") == "ListApplications"

    tests = [
        RuleTest(
            name="Test-291327",
            expected_result=True,
            log={
                "eventName": "ListApplications",
                "eventSource": "sso.amazonaws.com",
                "eventTime": "...",
                "eventVersion": "1.08",
                "sourceIPAddress": "<Victim source IP>",
                "userAgent": "<Victim browser user agent>",
                "userIdentity": {
                    "accountId": "<organization master account ID>",
                    "principalId": "<internal victim user id>",
                    "type": "Unknown",
                    "userName": "<victim display name>",
                },
            },
        ),
    ]
